import collections
import inspect
import operator
from abjad.system.AbjadValueObject import AbjadValueObject


class Pattern(AbjadValueObject):
    """
    Pattern.

    ..  container:: example

        Matches three indices out of every eight:

        >>> pattern = abjad.Pattern(
        ...     indices=[0, 1, 7],
        ...     period=8,
        ...     )

        >>> total_length = 16
        >>> for index in range(16):
        ...     match = pattern.matches_index(index, total_length)
        ...     match = match or ''
        ...     print(index, match)
        0 True
        1 True
        2
        3
        4
        5
        6
        7 True
        8 True
        9 True
        10
        11
        12
        13
        14
        15 True

    ..  container:: example

        Matches three indices out of every sixteen:

        >>> pattern = abjad.Pattern(
        ...     indices=[0, 1, 7],
        ...     period=16,
        ...     )

        >>> total_length = 16
        >>> for index in range(16):
        ...     match = pattern.matches_index(index, total_length)
        ...     match = match or ''
        ...     print(index, match)
        0 True
        1 True
        2
        3
        4
        5
        6
        7 True
        8
        9
        10
        11
        12
        13
        14
        15

    ..  container:: example

        Works with improper indices:

        >>> pattern = abjad.Pattern(
        ...     indices=[16, 17, 23],
        ...     period=16,
        ...     )

        >>> total_length = 16
        >>> for index in range(16):
        ...     match = pattern.matches_index(index, total_length)
        ...     match = match or ''
        ...     print(index, match)
        0 True
        1 True
        2
        3
        4
        5
        6
        7 True
        8
        9
        10
        11
        12
        13
        14
        15

    ..  container:: example

        Sieve from opening of Xenakis's Psappha:

        >>> sieve_1a = abjad.index([0, 1, 7], 8)
        >>> sieve_1b = abjad.index([1, 3], 5)
        >>> sieve_1 = sieve_1a & sieve_1b
        >>> sieve_2a = abjad.index([0, 1, 2], 8)
        >>> sieve_2b = abjad.index([0], 5)
        >>> sieve_2 = sieve_2a & sieve_2b
        >>> sieve_3 = abjad.index([3], 8)
        >>> sieve_4 = abjad.index([4], 8)
        >>> sieve_5a = abjad.index([5, 6], 8)
        >>> sieve_5b = abjad.index([2, 3, 4], 5)
        >>> sieve_5 = sieve_5a & sieve_5b
        >>> sieve_6a = abjad.index([1], 8)
        >>> sieve_6b = abjad.index([2], 5)
        >>> sieve_6 = sieve_6a & sieve_6b
        >>> sieve_7a = abjad.index([6], 8)
        >>> sieve_7b = abjad.index([1], 5)
        >>> sieve_7 = sieve_7a & sieve_7b
        >>> sieve = sieve_1 | sieve_2 | sieve_3 | sieve_4 | sieve_5 | sieve_6 | sieve_7

        >>> sieve.get_boolean_vector(total_length=40)
        [1, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 1,
        1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0]

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_indices',
        '_inverted',
        '_operator',
        '_patterns',
        '_payload',
        '_period',
        '_template',
        )

    _name_to_operator = {
        'and': operator.and_,
        'or': operator.or_,
        'xor': operator.xor,
        }

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        indices=None,
        inverted=None,
        operator=None,
        patterns=None,
        payload=None,
        period=None,
        template=None,
        ):
        import abjad
        if indices is not None:
            assert all(isinstance(_, int) for _ in indices), repr(indices)
            indices = tuple(indices)
        self._indices = indices
        if inverted is not None:
            inverted = bool(inverted)
        self._inverted = inverted
        if operator is not None:
            assert operator in self._name_to_operator, repr(operator)
        self._operator = operator
        if period is not None:
            assert abjad.mathtools.is_positive_integer(period), repr(period)
        if patterns is not None:
            assert all(isinstance(_, type(self)) for _ in patterns)
            patterns = tuple(patterns)
        self._patterns = patterns
        self._payload = payload
        self._period = period
        self._template = template

    ### SPECIAL METHODS ###

    def __and__(self, pattern):
        """
        Logical AND of two patterns.

        ..  container:: example

            Flat grouping of two patterns:

            >>> pattern_1 = abjad.index_first(3)
            >>> pattern_2 = abjad.index_last(3)
            >>> pattern = pattern_1 & pattern_2

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='and',
                patterns=(
                    abjad.index_first(3),
                    abjad.index_last(3),
                    ),
                )

        ..  container:: example

            Flat grouping of three patterns:

            >>> pattern_1 = abjad.index_first(3)
            >>> pattern_2 = abjad.index_last(3)
            >>> pattern_3 = abjad.index([0], 2)
            >>> pattern = pattern_1 & pattern_2 & pattern_3

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='and',
                patterns=(
                    abjad.index_first(3),
                    abjad.index_last(3),
                    abjad.index([0], period=2),
                    ),
                )

            >>> pattern.get_boolean_vector(total_length=16)
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]

        ..  container:: example

            Nested grouping of three patterns:

            >>> pattern_1 = abjad.index_first(3)
            >>> pattern_2 = abjad.index_last(3)
            >>> pattern_3 = abjad.index([0], 2)
            >>> pattern = pattern_1 & pattern_2 | pattern_3

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='or',
                patterns=(
                    abjad.Pattern(
                        operator='and',
                        patterns=(
                            abjad.index_first(3),
                            abjad.index_last(3),
                            ),
                        ),
                    abjad.index([0], period=2),
                    ),
                )

            >>> pattern.get_boolean_vector(total_length=16)
            [1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0]

        ..  container:: example

            In-place AND is allowed:

            >>> pattern = abjad.index_first(3)
            >>> pattern &= abjad.index_last(3)

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='and',
                patterns=(
                    abjad.index_first(3),
                    abjad.index_last(3),
                    ),
                )

        Returns new pattern.
        """
        if self._can_append_to_self(pattern, 'and'):
            if self.patterns is None:
                self_patterns = [self]
            else:
                self_patterns = list(self.patterns)
            patterns = self_patterns + [pattern]
            result = type(self)(operator='and', patterns=patterns)
        else:
            result = type(self)(operator='and', patterns=[self, pattern])
        return result

    def __invert__(self):
        """
        Inverts pattern.

        ..  container:: example

            >>> pattern = abjad.index_first(3)
            >>> abjad.f(pattern)
            abjad.index_first(3)

            >>> pattern = ~pattern
            >>> abjad.f(pattern)
            abjad.Pattern(
                indices=[0, 1, 2],
                inverted=True,
                )

            >>> pattern = ~pattern
            >>> abjad.f(pattern)
            abjad.Pattern(
                indices=[0, 1, 2],
                inverted=False,
                )

            Negation defined equal to inversion.

        ..  container:: example

            Matches every index that is (one of the first three indices) or
            (one of the last three indices):

            >>> pattern_1 = abjad.index_first(3)
            >>> pattern_2 = abjad.index_last(3)
            >>> pattern = pattern_1 | pattern_2

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='or',
                patterns=(
                    abjad.index_first(3),
                    abjad.index_last(3),
                    ),
                )

            >>> pattern.get_boolean_vector(total_length=16)
            [1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1]

        ..  container:: example

            Matches every index that is NOT (one of the first three indices) or
            (one of the last three indices):

            >>> pattern = ~pattern
            >>> abjad.f(pattern)
            abjad.Pattern(
                inverted=True,
                operator='or',
                patterns=(
                    abjad.index_first(3),
                    abjad.index_last(3),
                    ),
                )

            >>> pattern.get_boolean_vector(total_length=16)
            [0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0]

        Returns new pattern.
        """
        import abjad
        inverted = not self.inverted
        return abjad.new(self, inverted=inverted, template=None)

    def __len__(self):
        """
        Gets length of pattern.

        ..  container:: example

            Gets length of cyclic pattern:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=8,
            ...     )

            >>> len(pattern)
            8

            Length of cyclic pattern defined equal to period of the pattern.

        ..  container:: example

            Gets length of acyclic pattern:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 2, 3],
            ...     )

            >>> len(pattern)
            4

            Length of acyclic pattern defined equal to greatest index in
            pattern, plus 1.

        ..  container:: example

            Gets length of pattern with negative indices:

            >>> pattern = abjad.Pattern(
            ...     indices=[-3],
            ...     )

            >>> len(pattern)
            3

            Length of pattern with negative indices defined equal to absolute
            value of least index.

        Returns nonnegative integer.
        """
        if self.period is not None:
            return self.period
        if self.indices:
            absolute_indices = []
            for index in self.indices:
                if 0 <= index:
                    absolute_indices.append(index)
                else:
                    index = abs(index) - 1
                    absolute_indices.append(index)
            maximum_index = max(absolute_indices)
            return maximum_index + 1
        return 0

    def __or__(self, pattern):
        """
        Logical OR of two patterns.

        ..  container:: example

            Flat grouping of two patterns:

            >>> pattern_1 = abjad.index_first(3)
            >>> pattern_2 = abjad.index_last(3)
            >>> pattern = pattern_1 | pattern_2

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='or',
                patterns=(
                    abjad.index_first(3),
                    abjad.index_last(3),
                    ),
                )

        ..  container:: example

            Flat grouping:

            >>> pattern_1 = abjad.index_first(3)
            >>> pattern_2 = abjad.index_last(3)
            >>> pattern_3 = abjad.index([0], 2)
            >>> pattern = pattern_1 | pattern_2 | pattern_3

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='or',
                patterns=(
                    abjad.index_first(3),
                    abjad.index_last(3),
                    abjad.index([0], period=2),
                    ),
                )

            >>> pattern.get_boolean_vector(total_length=16)
            [1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1]

        ..  container:: example

            Nested grouping:

            >>> pattern_1 = abjad.index_first(3)
            >>> pattern_2 = abjad.index_last(3)
            >>> pattern_3 = abjad.index([0], 2)
            >>> pattern = pattern_1 | pattern_2 & pattern_3

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='or',
                patterns=(
                    abjad.index_first(3),
                    abjad.Pattern(
                        operator='and',
                        patterns=(
                            abjad.index_last(3),
                            abjad.index([0], period=2),
                            ),
                        ),
                    ),
                )

            >>> pattern.get_boolean_vector(total_length=16)
            [1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0]

        ..  container:: example

            In-place OR is allowed:

            >>> pattern = abjad.index_first(3)
            >>> pattern |= abjad.index_last(3)

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='or',
                patterns=(
                    abjad.index_first(3),
                    abjad.index_last(3),
                    ),
                )

        Returns new pattern.
        """
        if self._can_append_to_self(pattern, 'or'):
            if self.patterns is None:
                self_patterns = [self]
            else:
                self_patterns = list(self.patterns)
            patterns = self_patterns + [pattern]
            result = type(self)(operator='or', patterns=patterns)
        else:
            result = type(self)(operator='or', patterns=[self, pattern])
        return result

    def __xor__(self, pattern):
        """
        Logical XOR of two patterns.

        ..  container:: example

            >>> pattern_1 = abjad.index_first(3)
            >>> pattern_2 = abjad.index_last(3)
            >>> pattern = pattern_1 ^ pattern_2

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='xor',
                patterns=(
                    abjad.index_first(3),
                    abjad.index_last(3),
                    ),
                )

        ..  container:: example

            Flat grouping:

            >>> pattern_1 = abjad.index_first(3)
            >>> pattern_2 = abjad.index_last(3)
            >>> pattern_3 = abjad.index([0], 2)
            >>> pattern = pattern_1 ^ pattern_2 ^ pattern_3

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='xor',
                patterns=(
                    abjad.index_first(3),
                    abjad.index_last(3),
                    abjad.index([0], period=2),
                    ),
                )

            >>> pattern.get_boolean_vector(total_length=16)
            [0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1]

        ..  container:: example

            Nested grouping:

            >>> pattern_1 = abjad.index_first(3)
            >>> pattern_2 = abjad.index_last(3)
            >>> pattern_3 = abjad.index([0], 2)
            >>> pattern = pattern_1 ^ pattern_2 & pattern_3

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='xor',
                patterns=(
                    abjad.index_first(3),
                    abjad.Pattern(
                        operator='and',
                        patterns=(
                            abjad.index_last(3),
                            abjad.index([0], period=2),
                            ),
                        ),
                    ),
                )

            >>> pattern.get_boolean_vector(total_length=16)
            [1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0]

        ..  container:: example

            In-place XOR is allowed:

            >>> pattern = abjad.index_first(3)
            >>> pattern ^= abjad.index_last(3)

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='xor',
                patterns=(
                    abjad.index_first(3),
                    abjad.index_last(3),
                    ),
                )

        Returns new pattern.
        """
        if self._can_append_to_self(pattern, 'xor'):
            if self.patterns is None:
                self_patterns = [self]
            else:
                self_patterns = list(self.patterns)
            patterns = self_patterns + [pattern]
            result = type(self)(operator='xor', patterns=patterns)
        else:
            result = type(self)(operator='xor', patterns=[self, pattern])
        return result

    ### PRIVATE METHODS ###

    def _can_append_to_self(self, pattern, operator_):
        if not isinstance(pattern, type(self)):
            return False
        if self.operator is None:
            return True
        if (self.operator == operator_ and
            (pattern.operator is None or
            (pattern.operator == self.operator))):
            return True
        return False

    def _get_format_specification(self):
        import abjad
        if self.template is None:
            return super()._get_format_specification()
        return abjad.FormatSpecification(
            client=self,
            repr_is_indented=False,
            storage_format_is_indented=False,
            storage_format_args_values=[self.template],
            storage_format_forced_override=self.template,
            storage_format_kwargs_names=(),
            )

    @staticmethod
    def _get_template(frame):
        import abjad
        try:
            frame_info = inspect.getframeinfo(frame)
            function_name = frame_info.function
            arguments = abjad.Expression._wrap_arguments(
                frame,
                static_class=Pattern,
                )
            template = 'abjad.{}({})'.format(function_name, arguments)
        finally:
            del frame
        return template

    ### PUBLIC PROPERTIES ###

    @property
    def indices(self):
        """
        Gets indices of pattern.

        ..  container:: example

            Matches three indices out of every eight:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=8,
            ...     )

            >>> pattern.indices
            [0, 1, 7]

        ..  container:: example

            Matches three indices out of every sixteen:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=16,
            ...     )

            >>> pattern.indices
            [0, 1, 7]

        Defaults to none.

        Set to integers or none.

        Returns integers or none.
        """
        if self._indices:
            return list(self._indices)

    @property
    def inverted(self):
        """
        Is true when pattern is inverted.

        ..  container:: example

            Matches three indices out of every eight:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=8,
            ...     )

            >>> pattern.inverted is None
            True

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2
            3
            4
            5
            6
            7 True
            8 True
            9 True
            10
            11
            12
            13
            14
            15 True

        ..  container:: example

            Pattern that rejects three indices from every eight; equivalently,
            pattern matches ``8-3=5`` indices out of every eight:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=8,
            ...     inverted=True
            ...     )

            >>> pattern.inverted
            True

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0
            1
            2 True
            3 True
            4 True
            5 True
            6 True
            7
            8
            9
            10 True
            11 True
            12 True
            13 True
            14 True
            15

        ..  container:: example

            Matches every index that is (one of the first three indices) OR
            (one of the last three indices):

            >>> pattern_1 = abjad.index_first(3)
            >>> pattern_2 = abjad.index_last(3)
            >>> pattern = pattern_1 | pattern_2
            >>> pattern.inverted is None
            True

            >>> pattern.get_boolean_vector(total_length=16)
            [1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1]


        ..  container:: example

            Matches every index that is NOT (one of the first three indices) OR
            (one of the last three indices):

            >>> pattern = abjad.new(pattern, inverted=True)
            >>> pattern.inverted
            True

            >>> pattern.get_boolean_vector(total_length=16)
            [0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0]

        Defaults to none.

        Set to true, false or none.

        Returns true, false or none.
        """
        return self._inverted

    @property
    def operator(self):
        """
        Gets operator of pattern.

        Set to string.

        Returns string.
        """
        return self._operator

    @property
    def patterns(self):
        """
        Gets paterns of pattern.

        Set to patterns or none.

        Returns tuple of patterns or none.
        """
        return self._patterns

    @property
    def payload(self):
        """
        Gets payload of pattern.

        ..  container:: example

            Pattern with string payload assigned to three of every eight
            indices:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     payload='Allegro non troppo',
            ...     period=8,
            ...     )

            >>> total_length = 10
            >>> for index in range(10):
            ...     match = pattern.matches_index(index, total_length)
            ...     if match:
            ...         payload = pattern.payload
            ...     else:
            ...         payload = ''
            ...     print(index, repr(payload))
            ...
            0 'Allegro non troppo'
            1 'Allegro non troppo'
            2 ''
            3 ''
            4 ''
            5 ''
            6 ''
            7 'Allegro non troppo'
            8 'Allegro non troppo'
            9 'Allegro non troppo'

        Defaults to none.

        Set to any object.

        Returns arbitrary object.
        """
        return self._payload

    @property
    def period(self):
        """
        Gets period of pattern.

        ..  container:: example

            Pattern with a period of eight:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=8,
            ...     )

            >>> pattern.period
            8

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2
            3
            4
            5
            6
            7 True
            8 True
            9 True
            10
            11
            12
            13
            14
            15 True

        ..  container:: example

            Same pattern with a period of sixteen:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=16,
            ...     )

            >>> pattern.period
            16

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2
            3
            4
            5
            6
            7 True
            8
            9
            10
            11
            12
            13
            14
            15

        ..  container:: example

            Gets period of pattern that indexs every fourth and fifth element:

            >>> pattern_1 = abjad.Pattern([0], period=4)
            >>> pattern_2 = abjad.Pattern([0], period=5)
            >>> pattern = pattern_1 | pattern_2

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='or',
                patterns=(
                    abjad.Pattern(
                        indices=[0],
                        period=4,
                        ),
                    abjad.Pattern(
                        indices=[0],
                        period=5,
                        ),
                    ),
                period=20,
                )

            >>> pattern.period
            20

        ..  container:: example

            Returns none when pattern contains acyclic parts:

            >>> pattern_1 = abjad.Pattern([0], period=4)
            >>> pattern_2 = abjad.Pattern([0])
            >>> pattern = pattern_1 | pattern_2

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='or',
                patterns=(
                    abjad.Pattern(
                        indices=[0],
                        period=4,
                        ),
                    abjad.Pattern(
                        indices=[0],
                        ),
                    ),
                )

            >>> pattern.period is None
            True

        Defaults to none.

        Set to positive integer or none.

        Returns positive integer or none.
        """
        import abjad
        if self._period is not None:
            return self._period
        if self.patterns:
            periods = [_.period for _ in self.patterns]
            if None not in periods:
                return abjad.mathtools.least_common_multiple(*periods)

    @property
    def template(self):
        """
        Get pattern template.

        Set to string or none.

        Returns string or none.
        """
        return self._template

    @property
    def weight(self):
        """
        Gets weight of pattern.

        ..  container:: example

            Gets weight of cyclic pattern:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=8,
            ...     )

            >>> pattern.weight
            3

        ..  container:: example

            Gets weight of acyclic pattern:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 2, 3],
            ...     )

            >>> pattern.weight
            3

        Weight defined equal to number of indices in pattern.

        Returns nonnegative integer.
        """
        return len(self.indices)

    ### PUBLIC METHODS ###

    @classmethod
    def from_vector(class_, vector):
        """
        Makes pattern from boolean ``vector``.

        ..  container:: example

            Matches three indices out of every five:

            >>> pattern = [1, 0, 0, 1, 1]
            >>> pattern = abjad.Pattern.from_vector(pattern)
            >>> abjad.f(pattern)
            abjad.Pattern(
                indices=[0, 3, 4],
                period=5,
                )

            >>> total_length = 10
            >>> for index in range(10):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1
            2
            3 True
            4 True
            5 True
            6
            7
            8 True
            9 True

        ..  container:: example

            Matches three indices out of every six:

            >>> pattern = [1, 0, 0, 1, 1, 0]
            >>> pattern = abjad.Pattern.from_vector(pattern)
            >>> abjad.f(pattern)
            abjad.Pattern(
                indices=[0, 3, 4],
                period=6,
                )

            >>> total_length = 12
            >>> for index in range(12):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1
            2
            3 True
            4 True
            5
            6 True
            7
            8
            9 True
            10 True
            11

        Returns pattern.
        """
        vector = [bool(_) for _ in vector]
        period = len(vector)
        indices = [i for i, x in enumerate(vector) if x]
        return class_(
            period=period,
            indices=indices,
            )

    def get_boolean_vector(self, total_length=None):
        """
        Gets boolean vector of pattern applied to input sequence with
        ``total_length``.

        ..  container:: example

            Gets boolean vector of acyclic pattern:

            >>> pattern = abjad.Pattern(
            ...     indices=[4, 5, 6, 7],
            ...     )


            >>> pattern.get_boolean_vector(4)
            [0, 0, 0, 0]

            >>> pattern.get_boolean_vector(8)
            [0, 0, 0, 0, 1, 1, 1, 1]

            >>> pattern.get_boolean_vector(16)
            [0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0]

            Sets total length to length of pattern when ``total_length`` is
            none:

            >>> pattern.get_boolean_vector()
            [0, 0, 0, 0, 1, 1, 1, 1]

        ..  container:: example

            Gets vector of cyclic pattern:

            >>> pattern = abjad.Pattern(
            ...     indices=[4, 5, 6, 7],
            ...     period=20,
            ...     )

            >>> pattern.get_boolean_vector(4)
            [0, 0, 0, 0]

            >>> pattern.get_boolean_vector(8)
            [0, 0, 0, 0, 1, 1, 1, 1]

            >>> pattern.get_boolean_vector(16)
            [0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0]

            Sets total length to length of pattern when ``total_length`` is
            none:

            >>> pattern.get_boolean_vector()
            [0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]

        ..  container:: example

            Gets vector of inverted pattern:

            >>> pattern = abjad.Pattern(
            ...     indices=[4, 5, 6, 7],
            ...     period=20,
            ...     )

            >>> pattern.get_boolean_vector(4)
            [0, 0, 0, 0]

            >>> pattern.get_boolean_vector(8)
            [0, 0, 0, 0, 1, 1, 1, 1]

            >>> pattern.get_boolean_vector(16)
            [0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0]

            Sets total length to length of pattern when ``total_length`` is
            none:

            >>> pattern.get_boolean_vector()
            [0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]

        ..  container:: example

            Two-part pattern with logical OR:

            >>> pattern = abjad.Pattern(
            ...     operator='or',
            ...     patterns=[
            ...         abjad.Pattern(
            ...             indices=[0, 1, 2],
            ...             ),
            ...         abjad.Pattern(
            ...             indices=[-3, -2, -1],
            ...             ),
            ...         ],
            ...     )

            >>> pattern.get_boolean_vector(4)
            [1, 1, 1, 1]

            >>> pattern.get_boolean_vector(8)
            [1, 1, 1, 0, 0, 1, 1, 1]

            >>> pattern.get_boolean_vector(16)
            [1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1]

            Matches every index that is (one of the first three indices) OR
            (one of the last three indices).

        ..  container:: example

            Two-part pattern with mixed periodic and inverted parts:

            >>> pattern = abjad.Pattern(
            ...     operator='and',
            ...     patterns=[
            ...         abjad.Pattern(
            ...             indices=[0],
            ...             period=2,
            ...             ),
            ...         abjad.Pattern(
            ...             indices=[-3, -2, -1],
            ...             inverted=True,
            ...             ),
            ...         ],
            ...     )

            >>> pattern.get_boolean_vector(4)
            [1, 0, 0, 0]

            >>> pattern.get_boolean_vector(8)
            [1, 0, 1, 0, 1, 0, 0, 0]

            >>> pattern.get_boolean_vector(16)
            [1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0]

            Matches every index that is (equal to 0 % 2) AND (not one of the
            last three indices).

        ..  container:: example

            Cyclic pattern that indexes every fourth and fifth item:

            >>> pattern_1 = abjad.Pattern([0], period=4)
            >>> pattern_2 = abjad.Pattern([0], period=5)
            >>> pattern = pattern_1 | pattern_2

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='or',
                patterns=(
                    abjad.Pattern(
                        indices=[0],
                        period=4,
                        ),
                    abjad.Pattern(
                        indices=[0],
                        period=5,
                        ),
                    ),
                period=20,
                )

            >>> pattern.get_boolean_vector(4)
            [1, 0, 0, 0]

            >>> pattern.get_boolean_vector(8)
            [1, 0, 0, 0, 1, 1, 0, 0]

            >>> pattern.get_boolean_vector(16)
            [1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1]

            Sets total length to period of pattern when ``total_length`` is
            none:

            >>> pattern.period
            20

            >>> pattern.get_boolean_vector()
            [1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0, 0]

            >>> pattern.period == len(pattern.get_boolean_vector())
            True

        Returns list of ones and zeroes.
        """
        total_length = total_length or len(self)
        boolean_vector = []
        for index in range(total_length):
            result = self.matches_index(index, total_length)
            boolean_vector.append(int(result))
        return boolean_vector

    def get_matching_items(self, sequence):
        """
        Gets maching items from sequence.

        ..  container:: example

            >>> pattern = abjad.Pattern(
            ...     indices=[4, 5, 6, 7],
            ...     )

            >>> pattern.get_matching_items('abcdefghijklmnopqrstuvwxyz')
            Sequence(['e', 'f', 'g', 'h'])

        ..  container:: example

            >>> pattern = abjad.Pattern(
            ...     indices=[8, 9],
            ...     period=10,
            ...     )

            >>> pattern.get_matching_items('abcdefghijklmnopqrstuvwxyz')
            Sequence(['i', 'j', 's', 't'])

        ..  container:: example

            >>> pattern = abjad.index_first(1) | abjad.index_last(2)

            >>> pattern.get_matching_items('abcdefghijklmnopqrstuvwxyz')
            Sequence(['a', 'y', 'z'])

        Returns new sequence.
        """
        import abjad
        assert isinstance(sequence, collections.Iterable), repr(sequence)
        length = len(sequence)
        items = []
        for i in range(length):
            if self.matches_index(i, length):
                item = sequence[i]
                items.append(item)
        return abjad.sequence(items=items)

    @staticmethod
    def index(indices, period=None, inverted=None):
        """
        Makes pattern that matches ``indices``.

        ..  container:: example

            Indexes item 2:

            >>> pattern = abjad.index([2])

            >>> abjad.f(pattern)
            abjad.index([2])

        ..  container:: example

            Indexes items 2, 3 and 5:

            >>> pattern = abjad.index([2, 3, 5])

            >>> abjad.f(pattern)
            abjad.index([2, 3, 5])

        Returns pattern.
        """
        assert all(isinstance(_, int) for _ in indices), repr(indices)
        indices = indices or []
        template = Pattern._get_template(inspect.currentframe())
        return Pattern(
            indices=indices,
            inverted=inverted,
            period=period,
            template=template,
            )

    @staticmethod
    def index_all(inverted=None):
        """
        Makes pattern that matches all indices.

        ..  container:: example

            Indexes all divisions for tie creation:

            >>> pattern = abjad.index_all()

            >>> abjad.f(pattern)
            abjad.index_all()

        Returns pattern.
        """
        template = Pattern._get_template(inspect.currentframe())
        return Pattern(
            indices=[0],
            inverted=inverted,
            period=1,
            template=template,
            )

    @staticmethod
    def index_first(n, inverted=None):
        """
        Makes pattern that matches the first ``n`` indices.

        ..  container:: example

            Indexes first division for tie creation:

            >>> pattern = abjad.index_first(1)

            >>> abjad.f(pattern)
            abjad.index_first(1)

        ..  container:: example

            Indexes first two items:

            >>> pattern = abjad.index_first(2)

            >>> abjad.f(pattern)
            abjad.index_first(2)

        ..  container:: example

            Indexes nothing:

            >>> pattern = abjad.index_first(0)

            >>> abjad.f(pattern)
            abjad.index_first(0)

        Returns pattern.
        """
        assert isinstance(n, int), repr(n)
        if 0 < n:
            indices = list(range(n))
        else:
            indices = None
        template = Pattern._get_template(inspect.currentframe())
        return Pattern(
            indices=indices,
            inverted=inverted,
            template=template,
            )

    @staticmethod
    def index_last(n, inverted=None):
        """
        Makes pattern that matches the last ``n`` indices.

        ..  container:: example

            Indexes last two items:

            >>> pattern = abjad.index_last(2)

            >>> abjad.f(pattern)
            abjad.index_last(2)

        ..  container:: example

            Indexes nothing:

            >>> pattern = abjad.index_last(0)

            >>> abjad.f(pattern)
            abjad.index_last(0)

        Returns pattern.
        """
        assert isinstance(n, int), repr(n)
        if 0 < n:
            start = -1
            stop = -n - 1
            stride = -1
            indices = list(reversed(range(start, stop, stride)))
        else:
            indices = None
        template = Pattern._get_template(inspect.currentframe())
        return Pattern(
            indices=indices,
            inverted=inverted,
            template=template,
            )

    def matches_index(self, index, total_length, rotation=None):
        """
        Is true when pattern matches ``index`` taken under ``total_length``.

        ..  container:: example

            Matches three indices out of every eight:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=8,
            ...     )

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2
            3
            4
            5
            6
            7 True
            8 True
            9 True
            10
            11
            12
            13
            14
            15 True

            Matches three indices out of every eight, offset ``1`` to the left:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=8,
            ...     )

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(
            ...         index,
            ...         total_length,
            ...         rotation=1,
            ...         )
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1
            2
            3
            4
            5
            6 True
            7 True
            8 True
            9
            10
            11
            12
            13
            14 True
            15 True

            Matches three indices out of every eight, offset ``2`` to the
            left:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=8,
            ...     )

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(
            ...         index,
            ...         total_length,
            ...         rotation=2,
            ...         )
            ...     match = match or ''
            ...     print(index, match)
            0
            1
            2
            3
            4
            5 True
            6 True
            7 True
            8
            9
            10
            11
            12
            13 True
            14 True
            15 True

        ..  container:: example

            Matches three indices out of every sixteen:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=16,
            ...     )

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2
            3
            4
            5
            6
            7 True
            8
            9
            10
            11
            12
            13
            14
            15

            Matches three indices out of every sixteen, offset ``1`` to the
            left:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=16,
            ...     )

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(
            ...         index,
            ...         total_length,
            ...         rotation=1,
            ...         )
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1
            2
            3
            4
            5
            6 True
            7
            8
            9
            10
            11
            12
            13
            14
            15 True

            Matches three indices out of every sixteen, offset ``2`` to the
            left:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=16,
            ...     )

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(
            ...         index,
            ...         total_length,
            ...         rotation=2,
            ...         )
            ...     match = match or ''
            ...     print(index, match)
            0
            1
            2
            3
            4
            5 True
            6
            7
            8
            9
            10
            11
            12
            13
            14 True
            15 True

        ..  container:: example

            Empty pattern:

            >>> pattern = abjad.Pattern()

            Total length 16:

            >>> total_length = 16
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0
            1
            2
            3
            4
            5
            6
            7
            8
            9
            10
            11
            12
            13
            14
            15

            Total length 8:

            >>> total_length = 8
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0
            1
            2
            3
            4
            5
            6
            7

            Total length 4:

            >>> total_length = 4
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0
            1
            2
            3

            Matches nothing.

        ..  container:: example

            Simple pattern:

            Logical OR:

            >>> pattern = abjad.Pattern(
            ...     operator='or',
            ...     patterns=[
            ...         abjad.Pattern(
            ...             indices=[0, 1, 2],
            ...             ),
            ...         ],
            ...     )
            >>> total_length = 16
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2 True
            3
            4
            5
            6
            7
            8
            9
            10
            11
            12
            13
            14
            15

            Logical AND:

            >>> pattern = abjad.Pattern(
            ...     operator='and',
            ...     patterns=[
            ...         abjad.Pattern(
            ...             indices=[0, 1, 2],
            ...             ),
            ...         ],
            ...     )
            >>> total_length = 16
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2 True
            3
            4
            5
            6
            7
            8
            9
            10
            11
            12
            13
            14
            15

            Logical XOR:

            >>> pattern = abjad.Pattern(
            ...     operator='xor',
            ...     patterns=[
            ...         abjad.Pattern(
            ...             indices=[0, 1, 2],
            ...             ),
            ...         ],
            ...     )
            >>> total_length = 16
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2 True
            3
            4
            5
            6
            7
            8
            9
            10
            11
            12
            13
            14
            15

            Matches every index that is (one of the first three indices).

            Ignores ``operator``.

        ..  container:: example

            Two-part pattern with logical OR:

            >>> pattern = abjad.Pattern(
            ...     operator='or',
            ...     patterns=[
            ...         abjad.Pattern(
            ...             indices=[0, 1, 2],
            ...             ),
            ...         abjad.Pattern(
            ...             indices=[-3, -2, -1],
            ...             ),
            ...         ],
            ...     )

            Total length 16:

            >>> total_length = 16
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2 True
            3
            4
            5
            6
            7
            8
            9
            10
            11
            12
            13 True
            14 True
            15 True

            Total length 8:

            >>> total_length = 8
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2 True
            3
            4
            5 True
            6 True
            7 True

            Total length 4:

            >>> total_length = 4
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2 True
            3 True

            Matches every index that is (one of the first three indices) OR
            (one of the last three indices).

        ..  container:: example

            Two-part pattern with logical AND:

            >>> pattern = abjad.Pattern(
            ...     operator='and',
            ...     patterns=[
            ...         abjad.Pattern(
            ...             indices=[0, 1, 2],
            ...             ),
            ...         abjad.Pattern(
            ...             indices=[-3, -2, -1],
            ...             ),
            ...         ],
            ...     )

            Total length 16:

            >>> total_length = 16
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0
            1
            2
            3
            4
            5
            6
            7
            8
            9
            10
            11
            12
            13
            14
            15

            Total length 8:

            >>> total_length = 8
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0
            1
            2
            3
            4
            5
            6
            7

            Total length 4:

            >>> total_length = 4
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0
            1 True
            2 True
            3

            Matches every index that is (one of the first three indices) AND
            (one of the last three indices).

        ..  container:: example

            Two-part pattern with logical XOR:

            >>> pattern = abjad.Pattern(
            ...     operator='xor',
            ...     patterns=[
            ...         abjad.Pattern(
            ...             indices=[0, 1, 2],
            ...             ),
            ...         abjad.Pattern(
            ...             indices=[-3, -2, -1],
            ...             ),
            ...         ],
            ...     )

            Total length 16:

            >>> total_length = 16
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2 True
            3
            4
            5
            6
            7
            8
            9
            10
            11
            12
            13 True
            14 True
            15 True

            Total length 8:

            >>> total_length = 8
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2 True
            3
            4
            5 True
            6 True
            7 True

            Total length 4:

            >>> total_length = 4
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1
            2
            3 True

            Matches every index that is (one of the first three indices) XOR
            (one of the last three indices).

        ..  container:: example

            Two-part pattern with mixed periodic and inverted parts:

            >>> pattern = abjad.Pattern(
            ...     operator='and',
            ...     patterns=[
            ...         abjad.Pattern(
            ...             indices=[0],
            ...             period=2,
            ...             ),
            ...         abjad.Pattern(
            ...             indices=[-3, -2, -1],
            ...             inverted=True,
            ...             ),
            ...         ],
            ...     )

            Total length 16:

            >>> total_length = 16
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1
            2 True
            3
            4 True
            5
            6 True
            7
            8 True
            9
            10 True
            11
            12 True
            13
            14
            15

            Total length 8:

            >>> total_length = 8
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1
            2 True
            3
            4 True
            5
            6
            7

            Total length 4:

            >>> total_length = 4
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1
            2
            3

            Matches every index that is (equal to 0 % 2) AND (not one of the
            last three indices).

        ..  container:: example

            Complex pattern with compound and simple parts:

            >>> pattern = abjad.Pattern(
            ...     operator='and',
            ...     patterns=[
            ...         abjad.Pattern(
            ...             indices=[0],
            ...             period=2,
            ...             ),
            ...         abjad.Pattern(
            ...             indices=[-3, -2, -1],
            ...             inverted=True,
            ...             ),
            ...         ],
            ...     )
            >>> pattern = abjad.Pattern(
            ...     operator='or',
            ...     patterns=[
            ...         pattern,
            ...         abjad.Pattern(
            ...             indices=[0, 1, 2],
            ...             ),
            ...         ],
            ...     )

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='or',
                patterns=(
                    abjad.Pattern(
                        operator='and',
                        patterns=(
                            abjad.Pattern(
                                indices=[0],
                                period=2,
                                ),
                            abjad.Pattern(
                                indices=[-3, -2, -1],
                                inverted=True,
                                ),
                            ),
                        ),
                    abjad.Pattern(
                        indices=[0, 1, 2],
                        ),
                    ),
                )

            Total length 16:

            >>> total_length = 16
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2 True
            3
            4 True
            5
            6 True
            7
            8 True
            9
            10 True
            11
            12 True
            13
            14
            15

            Total length 8:

            >>> total_length = 8
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2 True
            3
            4 True
            5
            6
            7

            Total length 4:

            >>> total_length = 4
            >>> for index in range(total_length):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2 True
            3

            Matches every index that is ((equal to 0 % 2) AND (not one of the
            last three indices)) OR is (one of the first three indices).

        Returns true or false.
        """
        if not self.patterns:
            assert 0 <= total_length
            if 0 <= index:
                nonnegative_index = index
            else:
                nonnegative_index = total_length - abs(index)
            inverted = bool(self.inverted)
            if not self.indices:
                return False ^ inverted
            if self.period is None:
                for index in self.indices:
                    if index < 0:
                        index = total_length - abs(index)
                    if index == nonnegative_index and index < total_length:
                        return True ^ inverted
            else:
                if rotation is not None:
                    nonnegative_index += rotation
                nonnegative_index = nonnegative_index % self.period
                for index in self.indices:
                    if index < 0:
                        index = total_length - abs(index)
                        index = index % self.period
                    if index == nonnegative_index and index < total_length:
                        return True ^ inverted
                    if ((index % self.period) == nonnegative_index and
                        (index % self.period < total_length)):
                        return True ^ inverted
            return False ^ inverted
        elif len(self.patterns) == 1:
            pattern = self.patterns[0]
            result = pattern.matches_index(
                index,
                total_length,
                rotation=rotation,
                )
        else:
            operator_ = self._name_to_operator[self.operator]
            pattern = self.patterns[0]
            result = pattern.matches_index(
                index,
                total_length,
                rotation=rotation,
                )
            for pattern in self.patterns[1:]:
                result_ = pattern.matches_index(
                    index,
                    total_length,
                    rotation=rotation,
                    )
                result = operator_(result, result_)
        if self.inverted:
            result = not(result)
        return result

    def reverse(self):
        """
        Reverses pattern.

        ..  container:: example

            Matches three indices out of every eight:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=8,
            ...     )

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2
            3
            4
            5
            6
            7 True
            8 True
            9 True
            10
            11
            12
            13
            14
            15 True

        ..  container:: example

            Reverses pattern:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=8,
            ...     )

            >>> pattern = pattern.reverse()
            >>> abjad.f(pattern)
            abjad.Pattern(
                indices=[-1, -2, -8],
                period=8,
                )

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1
            2
            3
            4
            5
            6 True
            7 True
            8 True
            9
            10
            11
            12
            13
            14 True
            15 True

        ..  container:: example

            Matches every index that is (equal to 0 % 2) AND (not one of the
            last three indices):

            >>> pattern = abjad.Pattern(
            ...     operator='and',
            ...     patterns=[
            ...         abjad.Pattern(
            ...             indices=[0],
            ...             period=2,
            ...             ),
            ...         abjad.Pattern(
            ...             indices=[-3, -2, -1],
            ...             inverted=True,
            ...             ),
            ...         ],
            ...     )

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='and',
                patterns=(
                    abjad.Pattern(
                        indices=[0],
                        period=2,
                        ),
                    abjad.Pattern(
                        indices=[-3, -2, -1],
                        inverted=True,
                        ),
                    ),
                )

            Reverses pattern:

            >>> pattern = pattern.reverse()
            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='and',
                patterns=(
                    abjad.Pattern(
                        indices=[-1],
                        period=2,
                        ),
                    abjad.Pattern(
                        indices=[2, 1, 0],
                        inverted=True,
                        ),
                    ),
                )

            New pattern matches every index that is (equal to -1 % 2) AND
            (not one of the first three indices).

        Returns new pattern.
        """
        import abjad
        if not self.patterns:
            indices = [-index - 1 for index in self.indices]
            return abjad.new(self, indices=indices)
        patterns = [_.reverse() for _ in self.patterns]
        return abjad.new(self, patterns=patterns)

    def rotate(self, n=0):
        """
        Rotates pattern by index ``n``.

        ..  container:: example

            Matches three indices out of every eight:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=8,
            ...     )

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2
            3
            4
            5
            6
            7 True
            8 True
            9 True
            10
            11
            12
            13
            14
            15 True

            Rotates pattern two elements to the right:

            >>> pattern = abjad.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=8,
            ...     )

            >>> pattern = pattern.rotate(n=2)
            >>> abjad.f(pattern)
            abjad.Pattern(
                indices=[2, 3, 9],
                period=8,
                )

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0
            1 True
            2 True
            3 True
            4
            5
            6
            7
            8
            9 True
            10 True
            11 True
            12
            13
            14
            15

        ..  container:: example

            Matches three indices out of every eight with negative indices:

            >>> pattern = abjad.Pattern(
            ...     indices=[-3, -2, -1],
            ...     period=8,
            ...     )

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0
            1
            2
            3
            4
            5 True
            6 True
            7 True
            8
            9
            10
            11
            12
            13 True
            14 True
            15 True

            Rotates pattern two elements to the right:

            >>> pattern = abjad.Pattern(
            ...     indices=[-3, -2, -1],
            ...     period=8,
            ...     )

            >>> pattern = pattern.rotate(n=2)
            >>> abjad.f(pattern)
            abjad.Pattern(
                indices=[-1, 0, 1],
                period=8,
                )

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2
            3
            4
            5
            6
            7 True
            8 True
            9 True
            10
            11
            12
            13
            14
            15 True

        ..  container:: example

            Matches every index that is (equal to 0 % 2) AND (not one of the
            last three indices):

            >>> pattern = abjad.Pattern(
            ...     operator='and',
            ...     patterns=[
            ...         abjad.Pattern(
            ...             indices=[0],
            ...             period=2,
            ...             ),
            ...         abjad.Pattern(
            ...             indices=[-3, -2, -1],
            ...             inverted=True,
            ...             ),
            ...         ],
            ...     )

            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='and',
                patterns=(
                    abjad.Pattern(
                        indices=[0],
                        period=2,
                        ),
                    abjad.Pattern(
                        indices=[-3, -2, -1],
                        inverted=True,
                        ),
                    ),
                )

            Rotates pattern two elements to the right:

            >>> pattern = pattern.rotate(n=2)
            >>> abjad.f(pattern)
            abjad.Pattern(
                operator='and',
                patterns=(
                    abjad.Pattern(
                        indices=[2],
                        period=2,
                        ),
                    abjad.Pattern(
                        indices=[-1, 0, 1],
                        inverted=True,
                        ),
                    ),
                )

            New pattern matches every index that is (equal to 2 % 2) AND (not
            the first, second or last index in the pattern).

        Returns new pattern.
        """
        import abjad
        if not self.patterns:
            indices = [index + n for index in self.indices]
            return abjad.new(self, indices=indices)
        patterns = [_.rotate(n=n) for _ in self.patterns]
        return abjad.new(self, patterns=patterns)



from .GraceContainer import GraceContainer


class AppoggiaturaContainer(GraceContainer):
    r"""
    Appoggiatura container.

    Appoggiaturas are played on the beat.

    LilyPond positions appoggiaturas immediately before main notes.

    LilyPond formats appoggiaturas with a slur but without a slashed
    stem.

    ..  container:: example

        Appoggiatura notes:

        >>> voice = abjad.Voice("c'4 d'4 e'4 f'4")
        >>> notes = [abjad.Note("c'16"), abjad.Note("d'16")]
        >>> appoggiatura_container = abjad.AppoggiaturaContainer(notes)
        >>> abjad.attach(appoggiatura_container, voice[1])
        >>> abjad.show(voice) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(voice)
            \new Voice
            {
                c'4
                \appoggiatura {
                    c'16
                    d'16
                }
                d'4
                e'4
                f'4
            }

    Fill appoggiatura containers with notes, rests or chords.

    Attach appoggiatura containers to notes, rests or chords.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Containers'

    __slots__ = ()

    ### PRIVATE METHODS ###

    def _format_open_brackets_slot(self, bundle):
        result = []
        result.append([('grace_brackets', 'open'), [r'\appoggiatura {']])
        return tuple(result)



import bisect
from abjad import system
from abjad.system.AbjadObject import AbjadObject


class TimespanInequality(AbjadObject):
    """
    Timespan inequality.

    ..  container:: example

        >>> template = 'timespan_2.start_offset < timespan_1.start_offset'
        >>> inequality = abjad.TimespanInequality(template)

        >>> inequality
        TimespanInequality('timespan_2.start_offset < timespan_1.start_offset')

    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Time relations'

    __slots__ = (
        '_template',
        )

    _publish_storage_format = True

    templates = (
        'offset == timespan.start',
        'offset < timespan.start',
        'offset <= timespan.start',
        'offset == timespan.stop',
        'offset < timespan.stop',
        'offset <= timespan.stop',
        'timespan.start == offset',
        'timespan.start < offset',
        'timespan.start <= offset',
        'timespan.stop == offset',
        'timespan.stop < offset',
        'timespan.stop <= offset',
        'timespan_1.start_offset == timespan_2.start_offset',
        'timespan_1.start_offset < timespan_2.start_offset',
        'timespan_1.start_offset <= timespan_2.start_offset',
        'timespan_1.start_offset == timespan_2.stop_offset',
        'timespan_1.start_offset < timespan_2.stop_offset',
        'timespan_1.start_offset <= timespan_2.stop_offset',
        'timespan_1.stop_offset == timespan_2.start_offset',
        'timespan_1.stop_offset < timespan_2.start_offset',
        'timespan_1.stop_offset <= timespan_2.start_offset',
        'timespan_1.stop_offset == timespan_2.stop_offset',
        'timespan_1.stop_offset < timespan_2.stop_offset',
        'timespan_1.stop_offset <= timespan_2.stop_offset',
        'timespan_2.start_offset == timespan_1.start_offset',
        'timespan_2.start_offset < timespan_1.start_offset',
        'timespan_2.start_offset <= timespan_1.start_offset',
        'timespan_2.start_offset == timespan_1.stop_offset',
        'timespan_2.start_offset < timespan_1.stop_offset',
        'timespan_2.start_offset <= timespan_1.stop_offset',
        'timespan_2.stop_offset == timespan_1.start_offset',
        'timespan_2.stop_offset < timespan_1.start_offset',
        'timespan_2.stop_offset <= timespan_1.start_offset',
        'timespan_2.stop_offset == timespan_1.stop_offset',
        'timespan_2.stop_offset < timespan_1.stop_offset',
        'timespan_2.stop_offset <= timespan_1.stop_offset',
        )

    ### INITIALIZER ###

    def __init__(self, template=None):
        template = template or \
            'timespan_1.start_offset < timespan_2.start_offset'
        assert template in self.templates, repr(template)
        self._template = template

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        """
        Formats inequality.

            >>> template = 'timespan_2.start_offset < timespan_1.start_offset'
            >>> inequality = abjad.TimespanInequality(template)
            >>> abjad.f(inequality)
            abjad.TimespanInequality('timespan_2.start_offset < timespan_1.start_offset')

        Returns string.
        """
        if format_specification in ('', 'storage'):
            return system.StorageFormatManager(self).get_storage_format()
        return str(self)

    ### PRIVATE METHODS ###

    def _find_index_ge(self, a, x):
        """
        Finds index of leftmost item greater than or equal to x.
        """
        i = bisect.bisect_left(a, x)
        if i != len(a):
            return i
        raise ValueError

    def _find_index_gt(self, a, x):
        """
        Finds index of leftmost value greater than x.
        """
        i = bisect.bisect_right(a, x)
        if i != len(a):
            return i
        raise ValueError

    def _find_index_le(self, a, x):
        """
        Finds index of rightmost value less than or equal to x.
        """
        i = bisect.bisect_right(a, x)
        if i:
            return i - 1
        raise ValueError

    def _find_index_lt(self, a, x):
        """
        Finds index of rightmost value less than x.
        """
        i = bisect.bisect_left(a, x)
        if i:
            return i - 1
        raise ValueError

    def _get_format_specification(self):
        return system.FormatSpecification(
            client=self,
            storage_format_args_values=[self.template],
            storage_format_is_indented=False,
            storage_format_kwargs_names=[],
            )

    def _index(self, a, x):
        """
        Finds index of leftmost value exactly equal to x.
        """
        i = bisect.bisect_left(a, x)
        if i != len(a) and a[i] == x:
            return i
        raise ValueError

    @staticmethod
    def _make_repr(offset):
        return 'Offset({}, {})'.format(*offset.pair)

    ### PUBLIC METHODS ###

    def evaluate(
        self,
        timespan_1_start_offset,
        timespan_1_stop_offset,
        timespan_2_start_offset,
        timespan_2_stop_offset,
        ):
        """
        Evalutes inequality.

        Returns true or false.
        """
        import abjad
        make_repr = self._make_repr
        template = self.template
        template = template.replace(
            'timespan_1.start_offset', make_repr(timespan_1_start_offset))
        template = template.replace(
            'timespan_1.stop_offset', make_repr(timespan_1_stop_offset))
        template = template.replace(
            'timespan_2.start_offset', make_repr(timespan_2_start_offset))
        template = template.replace(
            'timespan_2.stop_offset', make_repr(timespan_2_stop_offset))
        truth_value = eval(template, {'Offset': abjad.Offset})
        return truth_value

    def evaluate_offset_inequality(
        self,
        timespan_start,
        timespan_stop,
        offset,
        ):
        """
        Evalutes offset inequality.

        Returns true or false.
        """
        import abjad
        make_repr = self._make_repr
        template = self.template
        template = self.template
        template = template.replace(
            'timespan.start', make_repr(timespan_start))
        template = template.replace(
            'timespan.stop', make_repr(timespan_stop))
        template = template.replace(
            'offset', make_repr(offset))
        truth_value = eval(template, {'Offset': abjad.Offset})
        return truth_value

    def get_offset_indices(
        self,
        timespan_1,
        timespan_2_start_offsets,
        timespan_2_stop_offsets,
        ):
        """
        Gets offset indices of inequality.

        .. todo:: add example.

        Returns nonnegative integer pair.
        """
        inequality = self.template
        assert isinstance(inequality, str), repr(inequality)
        leftmost_index, rightmost_index = None, None

        # 1.a
        if inequality == \
            'timespan_1.start_offset == timespan_2.start_offset':
            try:
                leftmost_index = self._find_index(
                    timespan_2_start_offsets, timespan_1.start_offset)
                rightmost_index = leftmost_index + 1
            except ValueError:
                pass
        # 1.b
        elif inequality == \
            'timespan_1.start_offset < timespan_2.start_offset':
            try:
                leftmost_index = self._find_index_gt(
                    timespan_2_start_offsets, timespan_1.start_offset)
                rightmost_index = len(timespan_2_start_offsets)
            except ValueError:
                pass
        # 1.c
        elif inequality == \
            'timespan_1.start_offset <= timespan_2.start_offset':
            try:
                leftmost_index = self._find_index_ge(
                    timespan_2_start_offsets, timespan_1.start_offset)
                rightmost_index = len(timespan_2_start_offsets)
            except ValueError:
                pass
        # 2.a
        elif inequality == \
            'timespan_1.start_offset == timespan_2.stop_offset':
            try:
                leftmost_index = self._find_index(
                    timespan_2_stop_offsets, timespan_1.start_offset)
                rightmost_index = leftmost_index + 1
            except ValueError:
                pass
        # 2.b
        elif inequality == \
            'timespan_1.start_offset < timespan_2.stop_offset':
            try:
                leftmost_index = self._find_index_gt(
                    timespan_2_stop_offsets, timespan_1.start_offset)
                rightmost_index = len(timespan_2_stop_offsets)
            except ValueError:
                pass
        # 2.c
        elif inequality == \
            'timespan_1.start_offset <= timespan_2.stop_offset':
            try:
                leftmost_index = self._find_index_ge(
                    timespan_2_stop_offsets, timespan_1.start_offset)
                rightmost_index = len(timespan_2_stop_offsets)
            except ValueError:
                pass
        # 3.a
        elif inequality == \
            'timespan_1.stop_offset == timespan_2.start_offset':
            try:
                leftmost_index = self._find_index(
                    timespan_2_start_offsets, timespan_1.stop_offset)
                rightmost_index = leftmost_index + 1
            except ValueError:
                pass
        # 3.b
        elif inequality == \
            'timespan_1.stop_offset < timespan_2.start_offset':
            try:
                leftmost_index = self._find_index_gt(
                    timespan_2_start_offsets, timespan_1.stop_offset)
                rightmost_index = len(timespan_2_start_offsets)
            except ValueError:
                pass
        # 3.c
        elif inequality == \
            'timespan_1.stop_offset <= timespan_2.start_offset':
            try:
                leftmost_index = self._find_index_ge(
                    timespan_2_start_offsets, timespan_1.stop_offset)
                rightmost_index = len(timespan_2_start_offsets)
            except ValueError:
                pass
        # 4.a
        elif inequality == \
            'timespan_1.stop_offset == timespan_2.stop_offset':
            try:
                leftmost_index = self._find_index(
                    timespan_2_start_offsets, timespan_1.stop_offset)
                rightmost_index = leftmost_index + 1
            except ValueError:
                pass
        # 4.b
        elif inequality == \
            'timespan_1.stop_offset < timespan_2.stop_offset':
            try:
                leftmost_index = self._find_index_gt(
                    timespan_2_stop_offsets, timespan_1.stop_offset)
                rightmost_index = len(timespan_2_stop_offsets)
            except ValueError:
                pass
        # 4.c
        elif inequality == \
            'timespan_1.stop_offset <= timespan_2.stop_offset':
            try:
                leftmost_index = self._find_index_ge(
                    timespan_2_stop_offsets, timespan_1.stop_offset)
                rightmost_index = len(timespan_2_stop_offsets)
            except ValueError:
                pass
        # 5.a
        elif inequality == \
            'timespan_2.start_offset == timespan_1.start_offset':
            try:
                leftmost_index = self._find_index(
                    timespan_2_start_offsets, timespan_1.start_offset)
                rightmost_index = leftmost_index + 1
            except ValueError:
                pass
        # 5.b
        elif inequality == \
            'timespan_2.start_offset < timespan_1.start_offset':
            try:
                leftmost_index = 0
                rightmost_index = self._find_index_ge(
                    timespan_2_start_offsets, timespan_1.start_offset)
            except ValueError:
                pass
        # 5.c
        elif inequality == \
            'timespan_2.start_offset <= timespan_1.start_offset':
            try:
                leftmost_index = 0
                rightmost_index = self._find_index_gt(
                    timespan_2_start_offsets, timespan_1.start_offset)
            except ValueError:
                pass
        # 6.a
        elif inequality == \
            'timespan_2.start_offset == timespan_1.stop_offset':
            try:
                leftmost_index = self._find_index(
                    timespan_2_start_offsets, timespan_1.stop_offset)
                rightmost_index = leftmost_index + 1
            except ValueError:
                pass
        # 6.b
        elif inequality == \
            'timespan_2.start_offset < timespan_1.stop_offset':
            try:
                leftmost_index = 0
                rightmost_index = self._find_index_ge(
                    timespan_2_start_offsets, timespan_1.stop_offset)
            except ValueError:
                pass
        # 6.c
        elif inequality == \
            'timespan_2.start_offset <= timespan_1.stop_offset':
            try:
                leftmost_index = 0
                rightmost_index = self._find_index_gt(
                    timespan_2_start_offsets, timespan_1.stop_offset)
            except ValueError:
                pass
        # 7.a
        elif inequality == \
            'timespan_2.stop_offset == timespan_1.start_offset':
            try:
                leftmost_index = self._find_index(
                    timespan_2_stop_offsets, timespan_1.start_offset)
                rightmost_index = leftmost_index + 1
            except ValueError:
                pass
        # 7.b
        elif inequality == \
            'timespan_2.stop_offset < timespan_1.start_offset':
            try:
                leftmost_index = 0
                rightmost_index = self._find_index_ge(
                    timespan_2_stop_offsets, timespan_1.start_offset)
            except ValueError:
                pass
        # 7.c
        elif inequality == \
            'timespan_2.stop_offset <= timespan_1.start_offset':
            try:
                leftmost_index = 0
                rightmost_index = self._find_index_gt(
                    timespan_2_stop_offsets, timespan_1.start_offset)
            except ValueError:
                pass
        # 8.a
        elif inequality == \
            'timespan_2.stop_offset == timespan_1.stop_offset':
            try:
                leftmost_index = self._find_index(
                    timespan_2_stop_offsets, timespan_1.stop_offset)
                rightmost_index = leftmost_index + 1
            except ValueError:
                pass
        # 8.b
        elif inequality == \
            'timespan_2.stop_offset < timespan_1.stop_offset':
            try:
                leftmost_index = 0
                rightmost_index = self._find_index_ge(
                    timespan_2_stop_offsets, timespan_1.stop_offset)
            except ValueError:
                pass
        # 8.c
        elif inequality == \
            'timespan_2.stop_offset <= timespan_1.stop_offset':
            try:
                leftmost_index = 0
                rightmost_index = self._find_index_gt(
                    timespan_2_stop_offsets, timespan_1.stop_offset)
            except ValueError:
                pass
        else:
            raise ValueError(inequality)

        if leftmost_index is not None and rightmost_index is not None:
            return leftmost_index, rightmost_index
        else:
            return []

    ### PUBLIC PROPERTIES ###

    @property
    def template(self):
        """
        Gets template of inequality.

            >>> template = 'timespan_2.start_offset < timespan_1.start_offset'
            >>> inequality = abjad.TimespanInequality(template)
            >>> inequality.template
            'timespan_2.start_offset < timespan_1.start_offset'

        Returns string.
        """
        return self._template



import typing
from abjad import enums
from abjad import typings
from abjad.core.Component import Component
from abjad.core.Leaf import Leaf
from abjad.core.Selection import Selection
from abjad.indicators.StartTextSpan import StartTextSpan
from abjad.indicators.StopTextSpan import StopTextSpan
from abjad.lilypondnames.LilyPondGrobOverride import LilyPondGrobOverride
from abjad.markups import Markup
from abjad.system.Tag import Tag
from abjad.system.Wrapper import Wrapper
from abjad.top.attach import attach
from abjad.top.inspect import inspect
from abjad.top.new import new
from abjad.top.select import select
from abjad.utilities.Expression import Expression
from abjad.utilities.OrderedDict import OrderedDict
from .Spanner import Spanner


class TextSpanner(Spanner):
    r"""
    Text spanner.

    ..  note:: ``abjad.TextSpanner`` spanner is deprecated. Use the
        ``abjad.text_spanner()`` factory function instead.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 f'4")
        >>> spanner = abjad.TextSpanner()
        >>> abjad.attach(spanner, staff[:])
        >>> abjad.override(staff).text_spanner.staff_padding = 2.5
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                \override TextSpanner.staff-padding = #2.5
            }
            {
                c'4
                \startTextSpan
                d'4
                e'4
                f'4
                \stopTextSpan
            }

    ..  container:: example

        Raises exception on fewer than two leaves:

        >>> staff = abjad.Staff("c'4 d' e' f'")
        >>> spanner = abjad.TextSpanner()
        >>> abjad.attach(spanner, staff[:1])
        Traceback (most recent call last):
            ...
        Exception: TextSpanner()._attachment_test_all():
          Requires at least two leaves.
          Not just Note("c'4").

    ..  container:: example

        Raises exception on noncontiguous leaves:

        >>> staff = abjad.Staff("c'4 d' e' f'")
        >>> spanner = abjad.TextSpanner()
        >>> abjad.attach(spanner, staff[:1] + staff[-1:])
        Traceback (most recent call last):
            ...
        Exception: TextSpanner() leaves must be contiguous:
          abjad.Selection(
            [
                abjad.Note("c'4"),
                abjad.Note("f'4"),
                ]
            )

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_skip_attachment_test_all',
        )

    _start_command = r'\startTextSpan'

    _stop_command = r'\stopTextSpan'

    ### INITIALIZER ###

    def __init__(self) -> None:
        Spanner.__init__(self)
        self._skip_attachment_test_all = False

    ### PRIVATE METHODS ###

    def _attachment_test_all(self, argument):
        if self._skip_attachment_test_all:
            return True
        result = self._at_least_two_leaves(argument)
        if result is not True:
            return result
        leaves = select(argument).leaves()
        return True

    def _get_lilypond_format_bundle(self, component=None):
        bundle = self._get_basic_lilypond_format_bundle(component)
        if component is self[0]:
            strings = self._tweaked_start_command_strings()
            bundle.after.spanner_starts.extend(strings)
        if component is self[-1]:
            string = self._stop_command_string()
            bundle.after.spanner_stops.append(string)
        return bundle

def text_spanner(
    argument: typing.Union[Component, Selection],
    *,
    selector: typings.Selector = 'abjad.select().leaves()',
    start_text_span: StartTextSpan = None,
    stop_text_span: StopTextSpan = None,
    ) -> None:
    r"""
    Attaches text span indicators.

    ..  container:: example

        Single spanner:

        >>> staff = abjad.Staff("c'4 d' e' f'")
        >>> start_text_span = abjad.StartTextSpan(
        ...     left_text=abjad.Markup('pont.').upright(),
        ...     right_text=abjad.Markup('tasto').upright(),
        ...     style='solid_line_with_arrow',
        ...     )
        >>> abjad.text_spanner(staff[:], start_text_span=start_text_span)
        >>> abjad.override(staff[0]).text_spanner.staff_padding = 4
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \once \override TextSpanner.staff-padding = #4
                c'4
                - \abjad_solid_line_with_arrow
                - \tweak bound-details.left.text \markup {
                    \concat
                        {
                            \upright
                                pont.
                            \hspace
                                #0.5
                        }
                    }
                - \tweak bound-details.right.text \markup {
                    \upright
                        tasto
                    }
                \startTextSpan
                d'4
                e'4
                f'4
                \stopTextSpan
            }

    ..  container:: example

        Enchained spanners:

        >>> staff = abjad.Staff("c'4 d' e' f' r")
        >>> start_text_span = abjad.StartTextSpan(
        ...     left_text=abjad.Markup('pont.').upright(),
        ...     style='dashed_line_with_arrow',
        ...     )
        >>> abjad.text_spanner(staff[:3], start_text_span=start_text_span)
        >>> start_text_span = abjad.StartTextSpan(
        ...     left_text=abjad.Markup('tasto').upright(),
        ...     right_text=abjad.Markup('pont.').upright(),
        ...     style='dashed_line_with_arrow',
        ...     )
        >>> abjad.text_spanner(staff[-3:], start_text_span=start_text_span)
        >>> abjad.override(staff).text_spanner.staff_padding = 4
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                \override TextSpanner.staff-padding = #4
            }
            {
                c'4
                - \abjad_dashed_line_with_arrow
                - \tweak bound-details.left.text \markup {
                    \concat
                        {
                            \upright
                                pont.
                            \hspace
                                #0.5
                        }
                    }
                \startTextSpan
                d'4
                e'4
                \stopTextSpan
                - \abjad_dashed_line_with_arrow
                - \tweak bound-details.left.text \markup {
                    \concat
                        {
                            \upright
                                tasto
                            \hspace
                                #0.5
                        }
                    }
                - \tweak bound-details.right.text \markup {
                    \upright
                        pont.
                    }
                \startTextSpan
                f'4
                r4
                \stopTextSpan
            }

        >>> staff = abjad.Staff("c'4 d' e' f' r")
        >>> start_text_span = abjad.StartTextSpan(
        ...     left_text=abjad.Markup('pont.').upright(),
        ...     style='dashed_line_with_arrow',
        ...     )
        >>> abjad.text_spanner(staff[:3], start_text_span=start_text_span)
        >>> start_text_span = abjad.StartTextSpan(
        ...     left_text=abjad.Markup('tasto').upright(),
        ...     style='solid_line_with_hook',
        ...     )
        >>> abjad.text_spanner(staff[-3:], start_text_span=start_text_span)
        >>> abjad.override(staff).text_spanner.staff_padding = 4
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                \override TextSpanner.staff-padding = #4
            }
            {
                c'4
                - \abjad_dashed_line_with_arrow
                - \tweak bound-details.left.text \markup {
                    \concat
                        {
                            \upright
                                pont.
                            \hspace
                                #0.5
                        }
                    }
                \startTextSpan
                d'4
                e'4
                \stopTextSpan
                - \abjad_solid_line_with_hook
                - \tweak bound-details.left.text \markup {
                    \concat
                        {
                            \upright
                                tasto
                            \hspace
                                #0.5
                        }
                    }
                \startTextSpan
                f'4
                r4
                \stopTextSpan
            }

    """
    import abjad
    start_text_span = start_text_span or StartTextSpan()
    stop_text_span = stop_text_span or StopTextSpan()

    if isinstance(selector, str):
        selector = eval(selector)
    assert isinstance(selector, Expression)
    argument = selector(argument)
    leaves = select(argument).leaves()
    start_leaf = leaves[0]
    stop_leaf = leaves[-1]
    
    attach(start_text_span, start_leaf)
    attach(stop_text_span, stop_leaf)



def mutate(client):
    r"""
    Makes mutation agent.

    ..  container:: example

        Scales duration of last note notes in staff:

        >>> staff = abjad.Staff("c'4 e'4 d'4 f'4")
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                e'4
                d'4
                f'4
            }

        >>> abjad.mutate(staff[-2:]).scale(abjad.Multiplier(3, 2))
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                e'4
                d'4.
                f'4.
            }

    ..  container:: example

        Returns mutation agent:

        >>> abjad.mutate(staff[-2:])
        Mutation(client=Selection([Note("d'4."), Note("f'4.")]))

    """
    import abjad
    return abjad.Mutation(client=client)



from .Leaf import Leaf


class Rest(Leaf):
    r"""
    Rest.

    ..  container:: example

        >>> rest = abjad.Rest('r8.')
        >>> measure = abjad.Measure((3, 16), [rest])
        >>> abjad.show(measure) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(measure)
            {   % measure
                \time 3/16
                r8.
            }   % measure

    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Leaves'

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, written_duration=None):
        import abjad
        original_input = written_duration
        if isinstance(written_duration, str):
            string = f'{{ {written_duration} }}'
            parsed = abjad.parse(string)
            assert len(parsed) == 1 and isinstance(parsed[0], Leaf)
            written_duration = parsed[0]
        if isinstance(written_duration, Leaf):
            written_duration = written_duration.written_duration
        elif written_duration is None:
            written_duration = abjad.Duration(1, 4)
        else:
            written_duration = abjad.Duration(written_duration)
        Leaf.__init__(self, written_duration)
        if isinstance(original_input, Leaf):
            self._copy_override_and_set_from_leaf(original_input)

    ### PRIVATE METHODS ###

    def _get_body(self):
        return [self._get_compact_representation()]

    def _get_compact_representation(self):
        return f'r{self._get_formatted_duration()}'



import collections
import inspect
from abjad.top.new import new
from abjad.utilities.Duration import Duration
from .Set import Set



class PitchSet(Set):
    r"""
    Pitch set.

    ..  container:: example

        Numbered pitch set:

        >>> set_ = abjad.PitchSet(
        ...     items=[-2, -1.5, 6, 7, -1.5, 7],
        ...     item_class=abjad.NumberedPitch,
        ...     )
        >>> set_
        PitchSet([-2, -1.5, 6, 7])

        >>> abjad.f(set_)
        abjad.PitchSet(
            [-2, -1.5, 6, 7]
            )

    ..  container:: example

        Named pitch set:

        >>> set_ = abjad.PitchSet(
        ...     ['bf,', 'aqs', "fs'", "g'", 'bqf', "g'"],
        ...     item_class=abjad.NamedPitch,
        ...     )
        >>> set_
        PitchSet(['bf,', 'aqs', 'bqf', "fs'", "g'"])

        >>> abjad.f(set_)
        abjad.PitchSet(
            ['bf,', 'aqs', 'bqf', "fs'", "g'"]
            )

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        """
        Is true when pitch set equals `argument`.

        ..  container:: example

            >>> set_1 = abjad.PitchSet(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     item_class=abjad.NumberedPitch,
            ...     )
            >>> set_2 = abjad.PitchSet(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     item_class=abjad.NumberedPitch,
            ...     )
            >>> set_3 = abjad.PitchSet(
            ...     items=[11, 12, 12.5],
            ...     item_class=abjad.NumberedPitch,
            ...     )

            >>> set_1 == set_1
            True
            >>> set_1 == set_2
            True
            >>> set_1 == set_3
            False

            >>> set_2 == set_1
            True
            >>> set_2 == set_2
            True
            >>> set_2 == set_3
            False

            >>> set_3 == set_1
            False
            >>> set_3 == set_2
            False
            >>> set_3 == set_3
            True

        Return true or false.
        """
        return super().__eq__(argument)

    def __hash__(self):
        """
        Hashes pitch set.

        Returns number.
        """
        return super().__hash__()

    def __illustrate__(self):
        r"""
        Illustrates pitch set.

        ..  container:: example

            Treble and bass pitches:

            >>> set_ = abjad.PitchSet(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     item_class=abjad.NumberedPitch,
            ...     )

            >>> abjad.show(set_) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = set_.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Score])
                \new Score
                <<
                    \new PianoStaff
                    <<
                        \new Staff
                        {
                            \new Voice
                            {
                                <fs' g'>1
                            }
                        }
                        \new Staff
                        {
                            \new Voice
                            {
                                <bf bqf>1
                            }
                        }
                    >>
                >>

        ..  container:: example

            Treble pitches only:

            >>> set_ = abjad.PitchSet(
            ...     items=[6, 7, 7],
            ...     item_class=abjad.NumberedPitch,
            ...     )

            >>> abjad.show(set_) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = set_.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Score])
                \new Score
                <<
                    \new PianoStaff
                    <<
                        \new Staff
                        {
                            \new Voice
                            {
                                <fs' g'>1
                            }
                        }
                        \new Staff
                        {
                            \new Voice
                            {
                                s1
                            }
                        }
                    >>
                >>

        Returns LilyPond file.
        """
        import abjad
        upper, lower = [], []
        for pitch in self:
            if pitch < 0:
                lower.append(pitch)
            else:
                upper.append(pitch)
        if upper:
            upper = abjad.Chord(upper, Duration(1))
        else:
            upper = abjad.Skip((1, 1))
        if lower:
            lower = abjad.Chord(lower, Duration(1))
        else:
            lower = abjad.Skip((1, 1))
        upper_voice = abjad.Voice([upper])
        upper_staff = abjad.Staff([upper_voice])
        lower_voice = abjad.Voice([lower])
        lower_staff = abjad.Staff([lower_voice])
        staff_group = abjad.StaffGroup(
            [upper_staff, lower_staff],
            lilypond_type='PianoStaff',
            )
        score = abjad.Score([staff_group])
        lilypond_file = abjad.LilyPondFile.new(score)
        return lilypond_file

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        import abjad
        return abjad.NamedPitch

    @property
    def _numbered_item_class(self):
        import abjad
        return abjad.NumberedPitch

    @property
    def _parent_item_class(self):
        import abjad
        return abjad.Pitch

    ### PRIVATE METHODS ###

    def _is_equivalent_under_transposition(self, argument):
        """
        True if pitch set is equivalent to `argument` under transposition.

        Returns true or false.
        """
        import abjad
        if not isinstance(argument, type(self)):
            return False
        if not len(self) == len(argument):
            return False
        difference = -(
            abjad.NamedPitch(argument[0], 4) -
            abjad.NamedPitch(self[0], 4)
            )
        new_pitches = (x + difference for x in self)
        new_pitches = new(self, items=new_pitches)
        return argument == new_pitches

    def _sort_self(self):
        import abjad
        return sorted(abjad.PitchSegment(tuple(self)))

    ### PUBLIC PROPERTIES ###

    @property
    def duplicate_pitch_classes(self):
        """
        Gets duplicate pitch-classes in pitch set.

        ..  container:: example

            >>> set_ = abjad.PitchSet(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     item_class=abjad.NumberedPitch,
            ...     )
            >>> set_.duplicate_pitch_classes
            PitchClassSet([])

            >>> set_ = abjad.PitchSet(
            ...     items=[-2, -1.5, 6, 7, 10.5, 7],
            ...     item_class=abjad.NumberedPitch,
            ...     )
            >>> set_.duplicate_pitch_classes
            PitchClassSet([10.5])

        Returns pitch-class set.
        """
        import abjad
        pitch_classes = []
        duplicate_pitch_classes = []
        for pitch in self:
            pitch_class = abjad.NumberedPitchClass(pitch)
            if pitch_class in pitch_classes:
                duplicate_pitch_classes.append(pitch_class)
            pitch_classes.append(pitch_class)
        return abjad.PitchClassSet(
            duplicate_pitch_classes,
            item_class=abjad.NumberedPitchClass,
            )

    @property
    def hertz(self):
        """
        Gets hertz of pitches in pitch segment.

        ..  container:: example

            >>> pitch_set = abjad.PitchSet('c e g b')
            >>> sorted(pitch_set.hertz)
            [130.81..., 164.81..., 195.99..., 246.94...]

        Returns set.
        """
        return set(_.hertz for _ in self)

    @property
    def is_pitch_class_unique(self):
        """
        Is true when pitch set is pitch-class-unique.

        ..  container:: example

            >>> set_ = abjad.PitchSet(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     item_class=abjad.NumberedPitch,
            ...     )

            >>> set_.is_pitch_class_unique
            True

        ..  container:: example

            >>> set_ = abjad.PitchSet(
            ...     items=[-2, -1.5, 6, 7, 10.5, 7],
            ...     item_class=abjad.NumberedPitch,
            ...     )

            >>> set_.is_pitch_class_unique
            False

        Returns true or false.
        """
        import abjad
        numbered_pitch_class_set = abjad.PitchClassSet(
            self, item_class=abjad.NumberedPitchClass)
        return len(self) == len(numbered_pitch_class_set)

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        """
        Makes pitch set from `selection`.

        ..  container:: example

            >>> staff_1 = abjad.Staff("c'4 <d' fs' a'>4 b2")
            >>> staff_2 = abjad.Staff("c4. r8 g2")
            >>> selection = abjad.select((staff_1, staff_2))
            >>> abjad.PitchSet.from_selection(selection)
            PitchSet(['c', 'g', 'b', "c'", "d'", "fs'", "a'"])

        Returns pitch set.
        """
        import abjad
        pitch_segment = abjad.PitchSegment.from_selection(selection)
        return class_(
            items=pitch_segment,
            item_class=item_class,
            )

    def invert(self, axis):
        """
        Inverts pitch set about `axis`.

        Returns new pitch set.
        """
        items = (pitch.invert(axis) for pitch in self)
        return new(self, items=items)

    def issubset(self, argument):
        """
        Is true when pitch set is subset of `argument`.

        ..  container:: example

            >>> set_1 = abjad.PitchSet(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     item_class=abjad.NumberedPitch,
            ...     )
            >>> set_2 = abjad.PitchSet(
            ...     items=[-1.5, 6],
            ...     item_class=abjad.NumberedPitch,
            ...     )

            >>> set_1.issubset(set_2)
            False

            >>> set_2.issubset(set_1)
            True

        Returns true or false.
        """
        return super().issubset(argument)

    def issuperset(self, argument):
        """
        Is true when pitch set is superset of `argument`.

        ..  container:: example

            >>> set_1 = abjad.PitchSet(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     item_class=abjad.NumberedPitch,
            ...     )
            >>> set_2 = abjad.PitchSet(
            ...     items=[-1.5, 6],
            ...     item_class=abjad.NumberedPitch,
            ...     )

            >>> set_1.issuperset(set_2)
            False

            >>> set_2.issuperset(set_1)
            True

        Returns true or false.
        """
        return super().issubset(argument)

    def register(self, pitch_classes):
        """
        Registers `pitch_classes` by pitch set.

        ..  container:: example

            >>> pitch_set = abjad.PitchSet(
            ...     items=[10, 19, 20, 23, 24, 26, 27, 29, 30, 33, 37, 40],
            ...     item_class=abjad.NumberedPitch,
            ...     )
            >>> pitch_classes = [10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11]
            >>> pitches = pitch_set.register(pitch_classes)
            >>> for pitch in pitches:
            ...     pitch
            NumberedPitch(10)
            NumberedPitch(24)
            NumberedPitch(26)
            NumberedPitch(30)
            NumberedPitch(20)
            NumberedPitch(19)
            NumberedPitch(29)
            NumberedPitch(27)
            NumberedPitch(37)
            NumberedPitch(33)
            NumberedPitch(40)
            NumberedPitch(23)

        Returns list of zero or more numbered pitches.
        """
        import abjad
        if isinstance(pitch_classes, collections.Iterable):
            result = [
                [_ for _ in self if _.number % 12 == pc]
                for pc in [x % 12 for x in pitch_classes]
                ]
            result = abjad.sequence(result).flatten(depth=-1)
        elif isinstance(pitch_classes, int):
            result = [p for p in pitch_classes if p % 12 == pitch_classes][0]
        else:
            message = 'must be pitch-class or list of pitch-classes.'
            raise TypeError(message)
        return result

    def transpose(self, n=0):
        """
        Transposes pitch set by index `n`.

        Returns new pitch set.
        """
        items = (pitch.transpose(n=n) for pitch in self)
        return new(self, items=items)



from abjad import mathtools
from abjad.system.FormatSpecification import FormatSpecification
from . import constants
from .Interval import Interval


class NumberedInterval(Interval):
    """
    Numbered interval.

    ..  container:: example

        Initializes from number of semitones:

        >>> abjad.NumberedInterval(-14)
        NumberedInterval(-14)

    ..  container:: example

        Initializes from other numbered interval

        >>> abjad.NumberedInterval(abjad.NumberedInterval(-14))
        NumberedInterval(-14)

    ..  container:: example

        Initializes from named interval:

        >>> abjad.NumberedInterval(abjad.NamedInterval('-P4'))
        NumberedInterval(-5)

    ..  container:: example

        Initializes from interval string:

        >>> abjad.NumberedInterval('-P4')
        NumberedInterval(-5)

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, number=0):
        super().__init__(number or 0)

    ### SPECIAL METHODS ###

    def __abs__(self):
        """
        Absolute value of numbered interval.

        ..  container:: example

            >>> abs(abjad.NumberedInterval(-14))
            NumberedInterval(14)

        Returns new numbered interval.
        """
        return type(self)(abs(self.number))

    def __add__(self, argument):
        """
        Adds `argument` to numbered interval.

        ..  container:: example

            >>> abjad.NumberedInterval(3) + abjad.NumberedInterval(14)
            NumberedInterval(17)

            >>> abjad.NumberedInterval(3) + abjad.NumberedInterval(-14)
            NumberedInterval(-11)

        Returns new numbered interval.
        """
        try:
            argument = type(self)(argument)
        except Exception:
            return NotImplemented
        return type(self)(float(self) + float(argument))

    def __copy__(self):
        """
        Copies numbered interval.

        >>> import copy

        ..  container:: example

            >>> copy.copy(abjad.NumberedInterval(-14))
            NumberedInterval(-14)

        Returns new numbered interval.
        """
        return type(self)(self.number)

    def __eq__(self, argument):
        """
        Is true when `argument` is a numbered interval with number equal to that of
        this numbered interval.

        ..  container:: example

            >>> interval_1 = abjad.NumberedInterval(12)
            >>> interval_2 = abjad.NumberedInterval(12)
            >>> interval_3 = abjad.NumberedInterval(13)

            >>> interval_1 == interval_1
            True
            >>> interval_1 == interval_2
            True
            >>> interval_1 == interval_3
            False

            >>> interval_2 == interval_1
            True
            >>> interval_2 == interval_2
            True
            >>> interval_2 == interval_3
            False

            >>> interval_3 == interval_1
            False
            >>> interval_3 == interval_2
            False
            >>> interval_3 == interval_3
            True

        Returns true or false.
        """
        return super().__eq__(argument)

    def __float__(self):
        """
        Coerce to float.

        Returns float.
        """
        return float(self.number)

    def __hash__(self):
        """
        Hashes numbered interval.

        Returns integer.
        """
        return super().__hash__()

    def __lt__(self, argument):
        """
        Is true when `argument` is a numbered interval with same direction
        number as this numbered interval and with number greater than that of
        this numbered interval.

        ..  container:: example

            >>> interval_1 = abjad.NumberedInterval(12)
            >>> interval_2 = abjad.NumberedInterval(12)
            >>> interval_3 = abjad.NumberedInterval(13)

            >>> interval_1 < interval_1
            False

            >>> interval_1 < interval_2
            False

            >>> interval_1 < interval_3
            True

            >>> interval_2 < interval_1
            False

            >>> interval_2 < interval_2
            False

            >>> interval_2 < interval_3
            True

            >>> interval_3 < interval_1
            False

            >>> interval_3 < interval_2
            False

            >>> interval_3 < interval_3
            False

        Returns true or false.
        """
        if not isinstance(argument, type(self)):
            message = 'must be numbered interval: {!r}.'
            message = message.format(argument)
            raise TypeError(message)
        if not self.direction_number == argument.direction_number:
            message = 'can only compare intervals of same direction.'
            raise ValueError(message)
        return abs(self.number) < abs(argument.number)

    def __neg__(self):
        """
        Negates numbered interval.

        ..  container:: example

            >>> -abjad.NumberedInterval(-14)
            NumberedInterval(14)

        Returns new numbered interval.
        """
        return type(self)(-self.number)

    def __radd__(self, argument):
        """
        Adds numbered interval to `argument`.

        ..  container:: example

            >>> interval = abjad.NumberedInterval(14)
            >>> abjad.NumberedInterval(3).__radd__(interval)
            NumberedInterval(17)

            >>> interval = abjad.NumberedInterval(-14)
            >>> abjad.NumberedInterval(3).__radd__(interval)
            NumberedInterval(-11)

        Returns new numbered interval.
        """
        try:
            argument = type(self)(argument)
        except Exception:
            return NotImplemented
        return type(self)(float(self) + float(argument))

    def __str__(self):
        """
        String representation of numbered interval.

        Returns string.
        """
        direction_symbol = constants._direction_number_to_direction_symbol[
            mathtools.sign(self.number)]
        return '{}{}'.format(direction_symbol, abs(self.number))

    def __sub__(self, argument):
        """
        Subtracts `argument` from numbered interval.

        Returns new numbered interval.
        """
        try:
            argument = type(self)(argument)
        except Exception:
            return NotImplemented
        return type(self)(float(self) - float(argument))

    ### PRIVATE METHODS ###

    def _from_named_parts(self, direction, quality, diatonic_number):
        self._from_number(self._named_to_numbered(
            direction,
            quality,
            diatonic_number,
            ))

    def _from_number(self, argument):
        import abjad
        number = self._to_nearest_quarter_tone(argument)
        direction = mathtools.sign(number)
        octaves = 0
        pc_number = abs(number)
        while pc_number > 12:
            pc_number -= 12
            octaves += 1
        self._octaves = octaves
        self._interval_class = abjad.NumberedIntervalClass(
            pc_number * direction)

    def _from_interval_or_interval_class(self, argument):
        self._from_number(float(argument))

    def _get_format_specification(self):
        values = [self.number]
        return FormatSpecification(
            client=self,
            coerce_for_equality=True,
            repr_is_indented=False,
            storage_format_is_indented=False,
            storage_format_args_values=values,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def direction_number(self):
        """
        Gets direction number of numbered interval.

        ..  container:: example

            >>> abjad.NumberedInterval(-14).direction_number
            -1

            >>> abjad.NumberedInterval(0).direction_number
            0

            >>> abjad.NumberedInterval(6).direction_number
            1

        Returns integer.
        """""
        return mathtools.sign(self.number)

    @property
    def interval_class(self):
        """
        Gets interval class of numbered interval.

        Returns numbered interval-class.
        """
        return self._interval_class

    @property
    def number(self):
        """
        Gets number of numbered interval.

        ..  container:: example

            >>> abjad.NumberedInterval(-14).number
            -14

            >>> abjad.NumberedInterval(-2).number
            -2

            >>> abjad.NumberedInterval(0).number
            0

        Returns number.
        """
        number = self._interval_class._number
        direction = mathtools.sign(number)
        number = abs(number) + (12 * self.octaves)
        return number * direction

    @property
    def octaves(self):
        """
        Gets octaves of interval.

        Returns nonnegative number.
        """
        return self._octaves

    @property
    def semitones(self):
        """
        Gets semitones corresponding to numbered interval.

        ..  container:: example

            >>> abjad.NumberedInterval(-14).semitones
            -14

        Returns nonnegative number.
        """
        return self.number

    ### PUBLIC METHODS ###

    @classmethod
    def from_pitch_carriers(class_, pitch_carrier_1, pitch_carrier_2):
        """Makes numbered interval from `pitch_carrier_1` and
        `pitch_carrier_2`.

        ..  container:: example

            >>> abjad.NumberedInterval.from_pitch_carriers(
            ...     abjad.NamedPitch(-2),
            ...     abjad.NamedPitch(12),
            ...     )
            NumberedInterval(14)

            >>> abjad.NumberedInterval.from_pitch_carriers(
            ...     abjad.NamedPitch(12),
            ...     abjad.NamedPitch(12),
            ...     )
            NumberedInterval(0)

            >>> abjad.NumberedInterval.from_pitch_carriers(
            ...     abjad.NamedPitch(9),
            ...     abjad.NamedPitch(12),
            ...     )
            NumberedInterval(3)

            >>> abjad.NumberedInterval.from_pitch_carriers(
            ...     abjad.NamedPitch(12),
            ...     abjad.NamedPitch(9),
            ...     )
            NumberedInterval(-3)

            >>> abjad.NumberedInterval.from_pitch_carriers(
            ...     abjad.NamedPitch(12),
            ...     abjad.NamedPitch(-2),
            ...     )
            NumberedInterval(-14)

        Returns numbered interval.
        """
        import abjad.pitch
        pitch_1 = abjad.pitch.NamedPitch(pitch_carrier_1)
        pitch_2 = abjad.pitch.NamedPitch(pitch_carrier_2)
        number = abjad.pitch.NumberedPitch(pitch_2).number - \
            abjad.pitch.NumberedPitch(pitch_1).number
        number = mathtools.integer_equivalent_number_to_integer(number)
        return class_(number)

    def transpose(self, pitch_carrier):
        """
        Transposes `pitch_carrier`.

        ..  container:: example

            Transposes chord:

            >>> chord = abjad.Chord("<c' e' g'>4")

            >>> interval = abjad.NumberedInterval(1)
            >>> interval.transpose(chord)
            Chord("<df' f' af'>4")

        Returns newly constructed object of `pitch_carrier` type.
        """
        return super().transpose(pitch_carrier)



#!/usr/bin/env python
import setuptools


if __name__ == '__main__':
    setuptools.setup(
        author='{composer_full_name}',
        author_email='{composer_email}',
        install_requires = (
            'abjad',
            ),
        name='{score_package_name}',
        packages=(
            '{score_package_name}',
            ),
        url='https://github.com/{composer_github_username}/{score_package_name}',
        version='0.1',
        zip_safe=False,
        )



lilypond_version = "2.16.1"

drums = {
    'acousticbassdrum': 'acousticbassdrum',
    'acousticsnare': 'acousticsnare',
    'agh': 'hiagogo',
    'agl': 'loagogo',
    'bassdrum': 'bassdrum',
    'bd': 'bassdrum',
    'bda': 'acousticbassdrum',
    'boh': 'hibongo',
    'bohm': 'mutehibongo',
    'boho': 'openhibongo',
    'bol': 'lobongo',
    'bolm': 'mutelobongo',
    'bolo': 'openlobongo',
    'cab': 'cabasa',
    'cabasa': 'cabasa',
    'cb': 'cowbell',
    'cgh': 'hiconga',
    'cghm': 'mutehiconga',
    'cgho': 'openhiconga',
    'cgl': 'loconga',
    'cglm': 'muteloconga',
    'cglo': 'openloconga',
    'chinesecymbal': 'chinesecymbal',
    'cl': 'claves',
    'claves': 'claves',
    'closedhihat': 'closedhihat',
    'cowbell': 'cowbell',
    'crashcymbal': 'crashcymbal',
    'crashcymbala': 'crashcymbala',
    'crashcymbalb': 'crashcymbalb',
    'cuim': 'mutecuica',
    'cuio': 'opencuica',
    'cymc': 'crashcymbal',
    'cymca': 'crashcymbala',
    'cymcb': 'crashcymbalb',
    'cymch': 'chinesecymbal',
    'cymr': 'ridecymbal',
    'cymra': 'ridecymbala',
    'cymrb': 'ridecymbalb',
    'cyms': 'splashcymbal',
    'da': 'onedown',
    'db': 'twodown',
    'dc': 'threedown',
    'dd': 'fourdown',
    'de': 'fivedown',
    'electricsnare': 'electricsnare',
    'fivedown': 'fivedown',
    'fiveup': 'fiveup',
    'fourdown': 'fourdown',
    'fourup': 'fourup',
    'gui': 'guiro',
    'guil': 'longguiro',
    'guiro': 'guiro',
    'guis': 'shortguiro',
    'halfopenhihat': 'halfopenhihat',
    'handclap': 'handclap',
    'hc': 'handclap',
    'hh': 'hihat',
    'hhc': 'closedhihat',
    'hhho': 'halfopenhihat',
    'hho': 'openhihat',
    'hhp': 'pedalhihat',
    'hiagogo': 'hiagogo',
    'hibongo': 'hibongo',
    'hiconga': 'hiconga',
    'highfloortom': 'highfloortom',
    'hightom': 'hightom',
    'hihat': 'hihat',
    'himidtom': 'himidtom',
    'hisidestick': 'hisidestick',
    'hitimbale': 'hitimbale',
    'hiwoodblock': 'hiwoodblock',
    'loagogo': 'loagogo',
    'lobongo': 'lobongo',
    'loconga': 'loconga',
    'longguiro': 'longguiro',
    'longwhistle': 'longwhistle',
    'losidestick': 'losidestick',
    'lotimbale': 'lotimbale',
    'lowfloortom': 'lowfloortom',
    'lowmidtom': 'lowmidtom',
    'lowoodblock': 'lowoodblock',
    'lowtom': 'lowtom',
    'mar': 'maracas',
    'maracas': 'maracas',
    'mutecuica': 'mutecuica',
    'mutehibongo': 'mutehibongo',
    'mutehiconga': 'mutehiconga',
    'mutelobongo': 'mutelobongo',
    'muteloconga': 'muteloconga',
    'mutetriangle': 'mutetriangle',
    'onedown': 'onedown',
    'oneup': 'oneup',
    'opencuica': 'opencuica',
    'openhibongo': 'openhibongo',
    'openhiconga': 'openhiconga',
    'openhihat': 'openhihat',
    'openlobongo': 'openlobongo',
    'openloconga': 'openloconga',
    'opentriangle': 'opentriangle',
    'pedalhihat': 'pedalhihat',
    'rb': 'ridebell',
    'ridebell': 'ridebell',
    'ridecymbal': 'ridecymbal',
    'ridecymbala': 'ridecymbala',
    'ridecymbalb': 'ridecymbalb',
    'shortguiro': 'shortguiro',
    'shortwhistle': 'shortwhistle',
    'sidestick': 'sidestick',
    'sn': 'snare',
    'sna': 'acousticsnare',
    'snare': 'snare',
    'sne': 'electricsnare',
    'splashcymbal': 'splashcymbal',
    'ss': 'sidestick',
    'ssh': 'hisidestick',
    'ssl': 'losidestick',
    'tamb': 'tambourine',
    'tambourine': 'tambourine',
    'threedown': 'threedown',
    'threeup': 'threeup',
    'timh': 'hitimbale',
    'timl': 'lotimbale',
    'tomfh': 'highfloortom',
    'tomfl': 'lowfloortom',
    'tomh': 'hightom',
    'toml': 'lowtom',
    'tommh': 'himidtom',
    'tomml': 'lowmidtom',
    'tri': 'triangle',
    'triangle': 'triangle',
    'trim': 'mutetriangle',
    'trio': 'opentriangle',
    'tt': 'tamtam',
    'twodown': 'twodown',
    'twoup': 'twoup',
    'ua': 'oneup',
    'ub': 'twoup',
    'uc': 'threeup',
    'ud': 'fourup',
    'ue': 'fiveup',
    'vibraslap': 'vibraslap',
    'vibs': 'vibraslap',
    'wbh': 'hiwoodblock',
    'wbl': 'lowoodblock',
    'whl': 'longwhistle',
    'whs': 'shortwhistle',
    }



#! /usr/bin/env python
import os
import setuptools


version_file_path = os.path.join(
    os.path.dirname(__file__),
    'abjad',
    '_version.py'
    )
with open(version_file_path, 'r') as file_pointer:
    file_contents_string = file_pointer.read()
local_dict = {}
exec(file_contents_string, None, local_dict)
__version__ = local_dict['__version__']

description = 'Abjad is a Python API for Formalized Score Control.'

with open('README.rst', 'r') as file_pointer:
    long_description = file_pointer.read()

author = [
    'Trevor Bača',
    'Josiah Wolf Oberholtzer',
    'Víctor Adán',
    ]
author = ', '.join(author)

author_email = [
    'trevor.baca@gmail.com',
    'josiah.oberholtzer@gmail.com',
    'contact@victoradan.net',
    ]
author_email = ', '.join(author_email)

classifiers = [
    'Development Status :: 5 - Production/Stable',
    'License :: OSI Approved :: GNU General Public License (GPL)',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: Implementation :: CPython',
    'Topic :: Artistic Software',
    ]

entry_points = {
    'console_scripts': [
        'ajv = abjad.cli:run_ajv',
        ]
    }

extras_require = {
    'accelerated': ['quicktions>=1.3'],
    'book': ['abjad-ext-book >= 3.0.0, < 3.1.0'],
    'cli': ['abjad-ext-cli >= 3.0.0, < 3.1.0'],
    'ipython': ['abjad-ext-ipython >= 3.0.0, < 3.1.0'],
    'nauert': ['abjad-ext-nauert >= 3.0.0, < 3.1.0'],
    'rmakers': ['abjad-ext-rmakers >= 3.0.0, < 3.1.0'],
    'tonality': ['abjad-ext-tonality >= 3.0.0, < 3.1.0'],
    'test': [
        'mypy',
        'pytest>=3.5.0',
        'pytest-cov',
        'pytest-helpers-namespace',
        ],
    }

keywords = [
    'music composition',
    'music notation',
    'formalized score control',
    'lilypond',
    ]
keywords = ', '.join(keywords)

install_requires = [
    'ply',
    'roman',
    'uqbar>=0.2.15',
    ]

if __name__ == '__main__':
    setuptools.setup(
        author=author,
        author_email=author_email,
        classifiers=classifiers,
        description=description,
        entry_points=entry_points,
        extras_require=extras_require,
        include_package_data=True,
        install_requires=install_requires,
        keywords=keywords,
        license='MIT',
        long_description=long_description,
        name='Abjad',
        packages=['abjad'],
        platforms='Any',
        url='http://www.projectabjad.org',
        version=__version__,
        )



from abjad import markups
from abjad.system.AbjadValueObject import AbjadValueObject


class Inversion(AbjadValueObject):
    """
    Inversion operator.

    ..  container:: example

        >>> abjad.Inversion()
        Inversion()

    ..  container:: example

        >>> abjad.Inversion(axis=15)
        Inversion(axis=NamedPitch("ef''"))

    Object model of twelve-tone inversion operator.
    """

    ### CLASS VARIABLES ##

    __slots__ = (
        '_axis',
        )

    ### INITIALIZER ###

    def __init__(self, *, axis=None):
        import abjad
        if axis is not None:
            axis = abjad.NamedPitch(axis)
        self._axis = axis

    ### SPECIAL METHODS ###

    def __add__(self, operator):
        r"""
        Composes inversion and `operator`.

        ..  container:: example

            Example segment:

            >>> items = [0, 2, 4, 5]
            >>> segment = abjad.PitchClassSegment(items=items)
            >>> abjad.show(segment) # doctest: +SKIP

            Example operators:

            >>> inversion = abjad.Inversion()
            >>> transposition = abjad.Transposition(n=3)

        ..  container:: example

            Transposition followed by inversion:

            >>> operator = inversion + transposition
            >>> str(operator)
            'IT3'

            >>> segment_ = operator(segment)
            >>> abjad.show(segment_) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment_.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    a'8
                    g'8
                    f'8
                    e'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Inversion followed by transposition:

            >>> operator = transposition + inversion
            >>> str(operator)
            'T3I'

            >>> segment_ = operator(segment)
            >>> abjad.show(segment_) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment_.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    ef'8
                    cs'8
                    b'8
                    bf'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Returns compound operator:

            >>> abjad.f(operator)
            abjad.CompoundOperator(
                operators=[
                    abjad.Inversion(),
                    abjad.Transposition(
                        n=3,
                        ),
                    ],
                )

        """
        import abjad
        return abjad.CompoundOperator._compose_operators(self, operator)

    def __call__(self, argument):
        """
        Calls inversion on `argument`.

        ..  container:: example

            Inverts numbered pitch-class:

            >>> inversion = abjad.Inversion()
            >>> pitch_class = abjad.NumberedPitchClass(1)
            >>> inversion(pitch_class)
            NumberedPitchClass(11)

        ..  container:: example

            Inverts numbered pitch:

            >>> inversion = abjad.Inversion()
            >>> pitch = abjad.NumberedPitch(15)
            >>> inversion(pitch)
            NumberedPitch(-15)

        ..  container:: example

            Inverts named pitch:

            >>> inversion = abjad.Inversion()
            >>> pitch = abjad.NamedPitch("d'")
            >>> inversion(pitch)
            NamedPitch('bf')

        ..  container:: example

            Inverts named pitch class:

            >>> inversion = abjad.Inversion()
            >>> pitch_class = abjad.NamedPitchClass('d')
            >>> inversion(pitch_class)
            NamedPitchClass('bf')

        ..  container:: example

            Inverts pitch segment:

            >>> inversion = abjad.Inversion()
            >>> segment = abjad.PitchSegment("c' d' e'")
            >>> inversion(segment)
            PitchSegment("c' bf af")

        ..  container:: example

            Inverts pitch class segment:

            >>> inversion = abjad.Inversion()
            >>> segment = abjad.PitchClassSegment("c d e")
            >>> inversion(segment)
            PitchClassSegment("c bf af")

        ..  container:: example

            Inverts pitch class set:

            >>> inversion = abjad.Inversion()
            >>> setting = abjad.PitchClassSet("c d e")
            >>> inversion(setting)
            PitchClassSet(['c', 'af', 'bf'])

        Returns new object with type equal to that of `argument`.
        """
        if hasattr(argument, 'invert'):
            result = argument.invert(axis=self.axis)
        else:
            message = 'do not know how to invert: {!r}.'
            message = message.format(argument)
            raise TypeError(message)
        return result

    def __radd__(self, operator):
        """
        Right-addition not defined on inversion.

        ..  container:: example

            >>> abjad.Inversion().__radd__(abjad.Inversion())
            Traceback (most recent call last):
                ...
            NotImplementedError: right-addition not defined on Inversion.

        Raises not implemented error.
        """
        message = 'right-addition not defined on {}.'
        message = message.format(type(self).__name__)
        raise NotImplementedError(message)

    def __str__(self):
        """
        Gets string representation of operator.

        ..  container:: example

            >>> str(abjad.Inversion())
            'I'

        ..  container:: example

            >>> str(abjad.Inversion(axis=15))
            'I(Eb5)'

        """
        if self.axis is None:
            return 'I'
        string = 'I({})'
        string = string.format(self.axis.get_name(locale='us'))
        return string

    ### PRIVATE METHODS ###

    def _get_markup(self, direction=None):
        markup = markups.Markup('I', direction=direction)
        if self.axis is not None:
            axis = self.axis.get_name(locale='us')
            subscript = markups.Markup(axis).sub()
            markup = markups.Markup.concat([markup, subscript])
        return markup

    def _is_identity_operator(self):
        return False

    ### PUBLIC PROPERTIES ###

    @property
    def axis(self):
        """
        Gets axis of inversion.

        ..  container:: example

            >>> inversion = abjad.Inversion()
            >>> inversion.axis is None
            True

        ..  container:: example

            >>> inversion = abjad.Inversion(axis=15)
            >>> inversion.axis
            NamedPitch("ef''")

        Returns named pitch or none.
        """
        return self._axis



from abjad.utilities import Infinity, NegativeInfinity
from .Timespan import Timespan


class AnnotatedTimespan(Timespan):
    """
    Annotated timespan.

    ..  container:: example

        >>> annotated_timespan = abjad.AnnotatedTimespan(
        ...    annotation=['a', 'b', 'c'],
        ...    start_offset=(1, 4),
        ...    stop_offset=(7, 8),
        ...    )
        >>> abjad.f(annotated_timespan)
        abjad.AnnotatedTimespan(
            start_offset=abjad.Offset(1, 4),
            stop_offset=abjad.Offset(7, 8),
            annotation=['a', 'b', 'c'],
            )

    Annotated timespans maintain their annotations duration mutation:

    ..  container:: example

        >>> left, right = annotated_timespan.split_at_offset((1, 2))
        >>> left.annotation.append('foo')
        >>> abjad.f(right)
        abjad.AnnotatedTimespan(
            start_offset=abjad.Offset(1, 2),
            stop_offset=abjad.Offset(7, 8),
            annotation=['a', 'b', 'c', 'foo'],
            )

    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Timespans'

    __slots__ = (
        '_annotation',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        start_offset=NegativeInfinity,
        stop_offset=Infinity,
        annotation=None,
        ):
        Timespan.__init__(
            self,
            start_offset=start_offset,
            stop_offset=stop_offset,
            )
        self._annotation = annotation

    ### PUBLIC PROPERTIES ###

    @property
    def annotation(self):
        """
        Gets and sets annotated timespan annotation.

        ..  container:: example

            Gets annotation:

            >>> annotated_timespan = abjad.AnnotatedTimespan(
            ...    annotation=['a', 'b', 'c', 'foo'],
            ...    start_offset=(1, 4),
            ...    stop_offset=(7, 8),
            ...    )
            >>> annotated_timespan.annotation
            ['a', 'b', 'c', 'foo']

        ..  container:: example

            Sets annotation:

            >>> annotated_timespan.annotation = 'baz'

        Returns arbitrary object.
        """
        return self._annotation

    @annotation.setter
    def annotation(self, argument):
        self._annotation = argument



import collections
import fractions
from abjad.system.AbjadValueObject import AbjadValueObject


class NonreducedRatio(AbjadValueObject, collections.Sequence):
    """
    Nonreduced ratio.

    ..  container:: example

        Nonreduced ratio of two numbers:

        >>> abjad.NonreducedRatio((2, 4))
        NonreducedRatio((2, 4))

    ..  container:: example

        Nonreduced ratio of three numbers:

            >>> abjad.NonreducedRatio((2, 4, 2))
            NonreducedRatio((2, 4, 2))

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_numbers',
        )

    ### INITIALIZER ###

    def __init__(self, numbers=(1, 1)):
        if isinstance(numbers, type(self)):
            numbers = numbers.numbers
        numbers = tuple(numbers)
        self._numbers = numbers

    ### SPECIAL METHODS ###

    def __contains__(self, argument):
        """
        Is true when ratio contains ``argument``.

        Returns true or false.
        """
        return argument in self._numbers

    def __eq__(self, argument):
        """
        Is true when ``argument`` is a nonreduced ratio with numerator and
        denominator equal to those of this nonreduced ratio.

        Returns true or false.
        """
        return super().__eq__(argument)

    def __format__(self, format_specification=''):
        """
        Formats duration.

        ..  container:: example

            >>> ratio = abjad.NonreducedRatio((2, 4, 2))
            >>> print(format(ratio))
            abjad.NonreducedRatio((2, 4, 2))

        Returns string.
        """
        import abjad
        if format_specification in ('', 'storage'):
            return abjad.StorageFormatManager(self).get_storage_format()
        return str(self)

    def __getitem__(self, argument):
        """
        Gets item or slice identified by ``argument``.

        ..  container:: example

            >>> ratio = abjad.NonreducedRatio((2, 4, 2))
            >>> ratio[1]
            4

        Returns integer or tuple.
        """
        if isinstance(argument, slice):
            return tuple(self._numbers.__getitem__(argument))
        return self._numbers.__getitem__(argument)

    def __hash__(self):
        """
        Hashes non-reduced ratio.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        """
        return super().__hash__()

    def __iter__(self):
        """
        Iterates ratio.

        Returns generator.
        """
        return iter(self._numbers)

    def __len__(self):
        """
        Gets length of ratio.

        ..  container:: example

            >>> ratio = abjad.NonreducedRatio((2, 4, 2))
            >>> len(ratio)
            3

        Returns integer.
        """
        return len(self._numbers)

    def __reversed__(self):
        """
        Iterates ratio in reverse.

        Returns generator.
        """
        return reversed(self._numbers)

    def __rtruediv__(self, number):
        """
        Divides ``number`` by ratio.

        ..  container:: example

            >>> 1 / abjad.Ratio((1, 1, 3))
            [Fraction(1, 5), Fraction(1, 5), Fraction(3, 5)]

        ..  container:: example

            >>> abjad.Fraction(1) / abjad.Ratio((1, 1, 3))
            [Fraction(1, 5), Fraction(1, 5), Fraction(3, 5)]

        ..  container:: example

            >>> 1.0 / abjad.Ratio((1, 1, 3))
            [0.2, 0.2, 0.6]

        Returns list of fractions or list of floats.
        """
        denominator = sum(self.numbers)
        factors = [fractions.Fraction(_, denominator) for _ in self.numbers]
        result = [_ * number for _ in factors]
        return result

    __rdiv__ = __rtruediv__

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        import abjad
        return abjad.FormatSpecification(
            client=self,
            storage_format_args_values=[self.numbers],
            storage_format_is_indented=False,
            storage_format_kwargs_names=[],
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _number_coercer(self):
        return int

    ### PUBLIC PROPERTIES ###

    @property
    def multipliers(self):
        """
        Gets multipliers of nonreduced ratio.

        ..  container:: example

            Nonreduced ratio of two numbers:

            >>> ratio = abjad.NonreducedRatio((2, 4))
            >>> ratio.multipliers
            (Multiplier(1, 3), Multiplier(2, 3))

        ..  container:: example

            Nonreduced ratio of three numbers:

            >>> ratio = abjad.NonreducedRatio((2, 4, 2))
            >>> ratio.multipliers
            (Multiplier(1, 4), Multiplier(1, 2), Multiplier(1, 4))

        Returns tuple of multipliers.
        """
        import abjad
        weight = sum(self.numbers)
        multipliers = [
            abjad.Multiplier((_, weight))
            for _ in self.numbers
            ]
        multipliers = tuple(multipliers)
        return multipliers

    @property
    def numbers(self):
        """
        Gets numbers of nonreduced ratio.

        ..  container:: example

            Nonreduced ratio of two numbers:

            >>> ratio = abjad.NonreducedRatio((2, 4))
            >>> ratio.numbers
            (2, 4)

        ..  container:: example

            Nonreduced ratio of three numbers:

            >>> ratio = abjad.NonreducedRatio((2, 4, 2))
            >>> ratio.numbers
            (2, 4, 2)

        Set to tuple of two or more numbers.

        Returns tuple of two or more numbers.
        """
        return self._numbers

    ### PUBLIC METHODS ###

    def count(self, argument):
        """
        Gets count of ``argument`` in ratio.

        Returns integer.
        """
        return self._numbers.count(argument)

    def index(self, argument):
        """
        Gets index of ``argument`` in ratio.

        Returns integer.
        """
        return self._numbers.index(argument)



import collections
from .TypedCollection import TypedCollection


class TypedList(TypedCollection, collections.MutableSequence):
    """
    Typed list.

    ..  container:: example

        No item coercion:

        >>> list_ = abjad.TypedList()
        >>> list_.append(23)
        >>> list_.append('foo')
        >>> list_.append(False)
        >>> list_.append((1, 2, 3))
        >>> list_.append(3.14159)

        >>> abjad.f(list_)
        abjad.TypedList(
            [
                23,
                'foo',
                False,
                (1, 2, 3),
                3.14159,
                ]
            )

    ..  container:: example

        Named pitch item coercion:

        >>> pitch_list = abjad.TypedList(
        ...     item_class=abjad.NamedPitch,
        ...     )
        >>> pitch_list.append(0)
        >>> pitch_list.append("d'")
        >>> pitch_list.append(('e', 4))
        >>> pitch_list.append(abjad.NamedPitch("f'"))

        >>> abjad.f(pitch_list)
        abjad.TypedList(
            [
                abjad.NamedPitch("c'"),
                abjad.NamedPitch("d'"),
                abjad.NamedPitch("e'"),
                abjad.NamedPitch("f'"),
                ],
            item_class=abjad.NamedPitch,
            )

    Ordered collection with optional item coercion.

    Implements the list interface.
    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_keep_sorted',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        items=None,
        item_class=None,
        keep_sorted=False,
        ):
        TypedCollection.__init__(
            self,
            item_class=item_class,
            items=items,
            )
        self._collection = []
        if keep_sorted is not None:
            assert isinstance(keep_sorted, bool), repr(keep_sorted)
        self._keep_sorted = keep_sorted
        items = items or []
        the_items = []
        for item in items:
            the_items.append(self._item_coercer(item))
        self.extend(the_items)

    ### SPECIAL METHODS ###

    def __delitem__(self, i):
        """
        Aliases list.__delitem__().

        Returns none.
        """
        self._on_removal(self._collection[i])
        del(self._collection[i])

    def __getitem__(self, argument):
        """
        Gets item or slice identified  by ``argument``.

        Returns item or slice.
        """
        return self._collection.__getitem__(argument)

    def __iadd__(self, argument):
        """
        Adds ``argument`` in place to typed list.

        ..  container:: example

            >>> dynamic_list = abjad.TypedList(item_class=abjad.Dynamic)
            >>> dynamic_list.append('ppp')
            >>> dynamic_list += ['p', 'mp', 'mf', 'fff']

            >>> abjad.f(dynamic_list)
            abjad.TypedList(
                [
                    abjad.Dynamic('ppp'),
                    abjad.Dynamic('p'),
                    abjad.Dynamic('mp'),
                    abjad.Dynamic('mf'),
                    abjad.Dynamic('fff'),
                    ],
                item_class=abjad.Dynamic,
                )

        Returns typed list.
        """
        self.extend(argument)
        return self

    def __reversed__(self):
        """
        Aliases list.__reversed__().

        Returns generator.
        """
        return self._collection.__reversed__()

    def __setitem__(self, i, argument):
        """
        Sets item ``i`` equal to ``argument``.

        ..  container:: example

            Sets item:

            >>> pitch_list = abjad.TypedList(
            ...     item_class=abjad.NamedPitch,
            ...     )
            >>> pitch_list.append(0)
            >>> pitch_list.append("d'")
            >>> pitch_list.append(('e', 4))
            >>> pitch_list.append(abjad.NamedPitch("f'"))

            >>> pitch_list[-1] = 'gqs,'
            >>> abjad.f(pitch_list)
            abjad.TypedList(
                [
                    abjad.NamedPitch("c'"),
                    abjad.NamedPitch("d'"),
                    abjad.NamedPitch("e'"),
                    abjad.NamedPitch('gqs,'),
                    ],
                item_class=abjad.NamedPitch,
                )

        ..  container:: example

            Sets slice:

            >>> pitch_list[-1:] = ["f'", "g'", "a'", "b'", "c''"]
            >>> abjad.f(pitch_list)
            abjad.TypedList(
                [
                    abjad.NamedPitch("c'"),
                    abjad.NamedPitch("d'"),
                    abjad.NamedPitch("e'"),
                    abjad.NamedPitch("f'"),
                    abjad.NamedPitch("g'"),
                    abjad.NamedPitch("a'"),
                    abjad.NamedPitch("b'"),
                    abjad.NamedPitch("c''"),
                    ],
                item_class=abjad.NamedPitch,
                )

        Returns none.
        """
        if isinstance(i, int):
            new_item = self._item_coercer(argument)
            old_item = self._collection[i]
            self._on_removal(old_item)
            self._on_insertion(new_item)
            self._collection[i] = new_item
        elif isinstance(i, slice):
            new_items = [self._item_coercer(item) for item in argument]
            old_items = self._collection[i]
            for old_item in old_items:
                self._on_removal(old_item)
            for new_item in new_items:
                self._on_insertion(new_item)
            self._collection[i] = new_items
        if self.keep_sorted:
            self.sort()

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        import abjad
        agent = abjad.StorageFormatManager(self)
        names = list(agent.signature_keyword_names)
        if 'items' in names:
            names.remove('items')
        if 'keep_sorted' in names:
            names.remove('keep_sorted')
        return abjad.FormatSpecification(
            self,
            repr_is_indented=False,
            storage_format_args_values=[self._collection],
            storage_format_kwargs_names=names,
            )

    ### PUBLIC METHODS ###

    def append(self, item):
        """
        Appends ``item`` to typed list.

        ..  container:: example

            >>> integer_list = abjad.TypedList(item_class=int)
            >>> integer_list.append('1')
            >>> integer_list.append(2)
            >>> integer_list.append(3.4)
            >>> integer_list[:]
            [1, 2, 3]

        Returns none.
        """
        item = self._item_coercer(item)
        self._on_insertion(item)
        self._collection.append(item)
        if self.keep_sorted:
            self.sort()

    def count(self, item):
        """
        Gets count of ``item`` in typed list.

        ..  container:: example

            >>> integer_list = abjad.TypedList(item_class=int)
            >>> integer_list.extend([0, 0., '0', 99])

            >>> integer_list.count(0)
            3
            >>> integer_list.count(1)
            0
            >>> integer_list.count(99)
            1

        Returns nonnegative integer.
        """
        item = self._item_coercer(item)
        return self._collection.count(item)

    def extend(self, items):
        """
        Extends typed list with ``items``.

        ..  container:: example

            >>> integer_list = abjad.TypedList(item_class=int)
            >>> integer_list.extend(['0', 1.0, 2, 3.14159])
            >>> integer_list
            TypedList([0, 1, 2, 3], item_class=int)

        Returns none.
        """
        for item in items:
            self.append(item)
        if self.keep_sorted:
            self.sort()

    def index(self, item):
        """
        Gets index of ``item`` in typed list.

        ..  container:: example

            >>> pitch_list = abjad.TypedList(
            ...     item_class=abjad.NamedPitch,
            ...     )
            >>> pitch_list.extend(['cqf', "as'", 'b,', 'dss'])

            >>> pitch_list.index(abjad.NamedPitch('cqf'))
            0
            >>> pitch_list.index(abjad.NamedPitch("as'"))
            1
            >>> pitch_list.index('b,')
            2
            >>> pitch_list.index('dss')
            3

        Returns nonnegative integer.
        """
        item = self._item_coercer(item)
        return self._collection.index(item)

    def insert(self, i, item):
        """
        Insert ``item`` into typed list.

        ..  container:: example

            Inserts into typed list.

            >>> integer_list = abjad.TypedList(item_class=int)
            >>> integer_list.extend(['1', 2, 4.3])
            >>> integer_list
            TypedList([1, 2, 4], item_class=int)

            >>> integer_list.insert(0, '0')
            >>> integer_list
            TypedList([0, 1, 2, 4], item_class=int)

            >>> integer_list.insert(1, '9')
            >>> integer_list
            TypedList([0, 9, 1, 2, 4], item_class=int)

        Returns none.
        """
        item = self._item_coercer(item)
        self._on_insertion(item)
        result = self._collection.insert(i, item)
        if self.keep_sorted:
            self.sort()
        return result

    def pop(self, i=-1):
        """
        Pops item ``i`` from typed list.

        Returns item.
        """
        result = self._collection.pop(i)
        self._on_removal(result)
        if self.keep_sorted:
            self.sort()
        return result

    def remove(self, item):
        """
        Removes ``item`` from typed list.

        ..  container:: example

            >>> integer_list = abjad.TypedList(item_class=int)
            >>> integer_list.extend(('0', 1.0, 2, 3.14159))
            >>> integer_list[:]
            [0, 1, 2, 3]

            >>> integer_list.remove('1')
            >>> integer_list[:]
            [0, 2, 3]

        Returns none.
        """
        item = self._item_coercer(item)
        index = self._collection.index(item)
        item = self._collection[index]
        self._on_removal(item)
        del(self._collection[index])
        if self.keep_sorted:
            self.sort()

    def reverse(self):
        """
        Reverses items in typed list.
        """
        self._collection.reverse()

    def sort(self, cmp=None, key=None, reverse=False):
        """
        Sorts items in typed list.
        """
        if cmp is not None:
            def cmp_to_key(comparator):
                """
                Convert a ``cmp`` function into a ``key`` function for use
                with ``sort()``.
                """
                class CmpToKey(object):

                    def __init__(self, argument):
                        self.argument = argument

                    def __lt__(self, other):
                        return comparator(self.argument, other.argument) < 0

                    def __gt__(self, other):
                        return comparator(self.argument, other.argument) > 0

                    def __eq__(self, other):
                        return comparator(self.argument, other.argument) == 0

                    def __le__(self, other):
                        return comparator(self.argument, other.argument) <= 0

                    def __ge__(self, other):
                        return comparator(self.argument, other.argument) >= 0

                    def __ne__(self, other):
                        return comparator(self.argument, other.argument) != 0

                return CmpToKey
            key = cmp_to_key(cmp)
        self._collection.sort(key=key, reverse=reverse)

    ### PUBLIC PROPERTIES ###

    @property
    def keep_sorted(self):
        """
        Is true when typed list keeps items sorted.

        Defaults to none.

        Set to true, false or none.

        Returns true, false or none.
        """
        return self._keep_sorted

    @keep_sorted.setter
    def keep_sorted(self, argument):
        assert isinstance(argument, (bool, type(None)))
        self._keep_sorted = argument



from abjad import Fraction


def is_nonnegative_integer_power_of_two(argument) -> bool:
    """
    Is true when ``argument`` is a nonnegative integer power of 2.

    ..  container:: example

        >>> for n in range(10):
        ...     print(n, abjad.mathtools.is_nonnegative_integer_power_of_two(n))
        ...
        0 True
        1 True
        2 True
        3 False
        4 True
        5 False
        6 False
        7 False
        8 True
        9 False

    """
    if isinstance(argument, int):
        return not bool(argument & (argument - 1))
    elif isinstance(argument, Fraction):
        return is_nonnegative_integer_power_of_two(
            argument.numerator * argument.denominator
            )
    else:
        return False



import abjad
import difflib
import ide
import os
import pathlib
import pytest
import shutil
import sys


abjad_ide = ide.AbjadIDE()
scores = pathlib.Path(*pathlib.Path(__file__).parts[:-4])
path = ide.Path(__file__, scores=scores)
directories = path.segments.list_paths()


@pytest.mark.parametrize('directory', directories)
def test_segments_01(directory):
    exit_code = abjad_ide.check_definition_py(directory)
    if exit_code != 0:
        sys.exit(exit_code)


@pytest.mark.parametrize('directory', directories)
def test_segments_02(directory):
    # only run on Travis because segment illustration usually takes a while
    if not os.getenv('TRAVIS'):
        return
    with abjad.FilesystemState(keep=[directory]):
        ly = directory / 'illustration.ly'
        ly_old = directory / 'illustration.old.ly'
        if ly.exists():
            shutil.copyfile(ly, ly_old)
        exit_code = abjad_ide.make_illustration_pdf(
            directory,
            open_after=False,
            )
        if exit_code != 0:
            sys.exit(exit_code)
        if not ly_old.exists():
            return
        assert ly.exists()
        assert ly_old.exists()
        if not abjad.TestManager.compare_files(ly_old, ly):
            ly_old_text = ly_old.read_text().splitlines(keepends=True)
            ly_text = ly.read_text().splitlines(keepends=True)
            print(''.join(difflib.ndiff(ly_old_text, ly_text)))
            sys.exit(1)



import shutil
import tempfile
from .ContextManager import ContextManager


class TemporaryDirectory(ContextManager):
    """
    A temporary directory context manager.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Context managers'

    __slots__ = (
        '_parent_directory',
        '_temporary_directory',
        )

    ### INITIALIZER ###

    def __init__(self, parent_directory=None):
        self._parent_directory = parent_directory
        self._temporary_directory = None

    ### SPECIAL METHODS ###

    def __enter__(self):
        """
        Enters context manager.

        Creates and returns path to a temporary directory.
        """
        self._temporary_directory = tempfile.mkdtemp(
            dir=self.parent_directory,
            )
        return self._temporary_directory

    def __exit__(self, exc_type, exc_value, traceback):
        """
        Exits context manager.

        Deletes previously created temporary directory.
        """
        shutil.rmtree(self._temporary_directory)

    ### PUBLIC PROPERTIES ###

    @property
    def parent_directory(self):
        """
        Gets parent directory.

        Returns string.
        """
        return self._parent_directory

    @property
    def temporary_directory(self):
        """
        Gets temporary directory.

        Returns string.
        """
        return self._temporary_directory
